package examples;

import java.net.*;
import java.util.*;
import java.io.*;
import java.text.*;

import com.ve.kavachart.utility.DataProvider;
import com.ve.kavachart.chart.Dataset;
import com.ve.kavachart.chart.Datum;

/**
 * A screen scraper class to get NBA standing data from USAToday
 */
public class ALSchedule implements DataProvider {

	String startTag = "<!-- =================== Begin Sports Content";
	String endTag = "<!-- =================== End Sports Content";
	public boolean noisy = false;
	Vector datasets = null;
	Date lastRetrieved = null;
	
	String[] teamLinks = {
		"http://anaheim.angels.mlb.com/NASApp/mlb/index.jsp?c_id=ana",
		"http://arizona.diamondbacks.mlb.com/NASApp/mlb/index.jsp?c_id=ari",
		"http://atlanta.braves.mlb.com/NASApp/mlb/index.jsp?c_id=atl",
		"http://baltimore.orioles.mlb.com/NASApp/mlb/index.jsp?c_id=bal",
		"http://boston.redsox.mlb.com/NASApp/mlb/index.jsp?c_id=bos",
		"http://chicago.cubs.mlb.com/NASApp/mlb/index.jsp?c_id=chc",
		"http://chicago.whitesox.mlb.com/NASApp/mlb/index.jsp?c_id=cws",
		"http://cincinnati.reds.mlb.com/NASApp/mlb/index.jsp?c_id=cin",
		"http://cleveland.indians.mlb.com/NASApp/mlb/index.jsp?c_id=cle",
		"http://colorado.rockies.mlb.com/NASApp/mlb/index.jsp?c_id=col",
		"http://detroit.tigers.mlb.com/NASApp/mlb/index.jsp?c_id=det",
		"http://florida.marlins.mlb.com/NASApp/mlb/index.jsp?c_id=fla",
		"http://houston.astros.mlb.com/NASApp/mlb/index.jsp?c_id=hou",
		"http://kansascity.royals.mlb.com/NASApp/mlb/index.jsp?c_id=kc",
		"http://losangeles.dodgers.mlb.com/NASApp/mlb/index.jsp?c_id=la",
		"http://milwaukee.brewers.mlb.com/NASApp/mlb/index.jsp?c_id=mil",
		"http://minnesota.twins.mlb.com/NASApp/mlb/index.jsp?c_id=min",
		"http://montreal.expos.mlb.com/mon/home.html",
		"http://newyork.mets.mlb.com/NASApp/mlb/index.jsp?c_id=nym",
		"http://newyork.yankees.mlb.com/NASApp/mlb/index.jsp?c_id=nyy",
		"http://oakland.athletics.mlb.com/NASApp/mlb/index.jsp?c_id=oak",
		"http://philadelphia.phillies.mlb.com/NASApp/mlb/index.jsp?c_id=phi",
		"http://pittsburgh.pirates.mlb.com/NASApp/mlb/index.jsp?c_id=pit",
		"http://sandiego.padres.mlb.com/NASApp/mlb/index.jsp?c_id=sd",
		"http://sanfrancisco.giants.mlb.com/NASApp/mlb/index.jsp?c_id=sf",
		"http://seattle.mariners.mlb.com/NASApp/mlb/index.jsp?c_id=sea",
		"http://stlouis.cardinals.mlb.com/NASApp/mlb/index.jsp?c_id=stl",
		"http://tampabay.devilrays.mlb.com/NASApp/mlb/index.jsp?c_id=tb",
		"http://texas.rangers.mlb.com/NASApp/mlb/index.jsp?c_id=tex",
		"http://toronto.bluejays.mlb.com/NASApp/mlb/index.jsp?c_id=tor"
	};

	String[] teamNames = {
		"ANAHEIM",
		"ARIZONA",
		"ATLANTA",
		"BALTIMORE",
		"BOSTON",
		"CHICAGO CUBS",
		"CHICAGO WSOX",
		"CINCINNATI",
		"CLEVELAND",
		"COLORADO",
		"DETROIT",
		"FLORIDA",
		"HOUSTON",
		"KANSAS CITY",
		"LOS ANGELES",
		"MILWAUKEE",
		"MINNESOTA",
		"MONTREAL",
		"NY METS",
		"NY YANKEES",
		"OAKLAND",
		"PHILADELPHIA",
		"PITTSBURGH",
		"SAN DIEGO",
		"SAN FRANCISCO",
		"SEATTLE",
		"ST LOUIS",
		"TAMPA BAY",
		"TEXAS",
		"TORONTO",
	};
	
	Hashtable links;
	/**
	 * This method does the work of retrieving the data from the BLS web site and converting it into
	 * KavaChart Dataset classes.  It's pretty simple, really.  Just a bunch of stuff to generate
	 * and parse the page.
	 */
	private void generateNL(){
		generateLeague("http://netsports.baseball.com/default.asp?c=baseball&page=mlb/stat/nl-yearly.htm");	
	}
	
	private void generateAL(){
		generateLeague("http://netsports.baseball.com/default.asp?c=baseball&page=mlb/stat/al-yearly.htm");
	}
		private void generateLeague(String urlString){

		Hashtable hashtable = new Hashtable();
		BufferedReader reader = null;
		try{
		URL alURL = new URL(urlString);
		reader = new BufferedReader(new InputStreamReader(alURL.openStream()));
		String line = reader.readLine(); 
		while(!line.startsWith(startTag)){ //strip off header
			line = reader.readLine();
		}
		
		//strip off table header
		for(int i=0;i<8;i++)
			line = reader.readLine();
			
		SimpleDateFormat sdf = new SimpleDateFormat(" EEE, MMM dd");
		Date schedDate;
		//start doing schedule...
		while(!line.startsWith(endTag)){
			//System.out.println("------");
			line = reader.readLine();
			schedDate = sdf.parse(line);
			schedDate.setYear(2004 - 1900);
			//System.out.println(schedDate);
			line = reader.readLine();
			while(line.length()>0){ //splits dates
				int hyphenIndex = line.indexOf("-");
				if(hyphenIndex!=-1){
					String winner = line.substring(1, hyphenIndex-1);
					int scoreIndex = winner.lastIndexOf(" ");
					String teamName = winner.substring(0, scoreIndex);
					String winnerName = teamName.toUpperCase();
					double winnerScore = Double.parseDouble(winner.substring(scoreIndex + 1));
					
					String loser = line.substring(hyphenIndex + 2, line.indexOf(","));
					scoreIndex = loser.lastIndexOf(" ");
					teamName = loser.substring(0, scoreIndex);
					String loserName = teamName.toUpperCase();
					double loserScore = Double.parseDouble(loser.substring(scoreIndex + 1));
					
					Datum winnerDatum = new Datum(schedDate.getTime(), winnerScore - loserScore, null);
					winnerDatum.setLabel("Defeated " + loserName);
					Datum loserDatum = new Datum(schedDate.getTime(), loserScore - winnerScore, null);
					loserDatum.setLabel("Lost To " + winnerName);
					
					ArrayList list = (ArrayList)hashtable.get(winnerName);
					if(list==null){
						list = new ArrayList();
						hashtable.put(winnerName, list);
					}
					list.add(winnerDatum);
					
					list = (ArrayList)hashtable.get(loserName);
					if(list==null){
						list = new ArrayList();
						hashtable.put(loserName, list);
					}
					list.add(loserDatum);
				}
				line = reader.readLine();
			}
		}
		
		}
		catch(Exception io){
			//io.printStackTrace();
			//return;
			try{
			reader.close();
			}catch(IOException ignored){
			}
		}

		Enumeration keys = hashtable.keys();
		while(keys.hasMoreElements()){
			String teamName = (String)keys.nextElement();
			
			//System.out.println(teamName);
			Dataset dataset = new Dataset();
			dataset.setName(teamName);
			ArrayList al = (ArrayList)hashtable.get(teamName);
			Object[] objs = al.toArray();
			for(int i=0;i<objs.length;i++){
				dataset.getData().addElement(objs[i]);
			}
			datasets.addElement(dataset);
			//System.out.println("number of games:" + objs.length);
		}
	}
	
	private void generate(){
		datasets = new Vector();
		generateNL();
		generateAL();
		//buildLinks();
	}
	
	private void buildLinks(){
		links = new Hashtable();
		for(int i=0;i<teamLinks.length;i++){
			links.put(teamNames[i], teamLinks[i]);
		}
	}
	
	public String getDatasetLinks(int setNum){
		if(datasets==null)
			getDatasets();
		StringBuffer sb = new StringBuffer();
		Dataset d = (Dataset)datasets.elementAt(setNum);
		Enumeration e = d.getData().elements();
		while(e.hasMoreElements()){
			Datum dat = (Datum)e.nextElement();
			sb.append(links.get(dat.getLabel()));
			sb.append(",");
		}
		return sb.toString();
	}
	
	final static String WON = "WON";
	final static String LOST = "LOST";

	private Dataset buildDataset(String division, Vector teams){
		Dataset d = new Dataset();
		d.setName(division);
		Enumeration e = teams.elements();
		while(e.hasMoreElements()){
			String team = (String)e.nextElement();
			//System.out.println(team + ", " + team.length());
			if(team.length()>1){
				String name = team.substring(0, 16).trim();
				team = team.substring(17);
				StringTokenizer st = new StringTokenizer(team);
				String wins = st.nextToken();
				String losses = st.nextToken();
				String percentage = st.nextToken();
				double streak = 0.0;
				int streakLoc = team.indexOf(WON);
				if(streakLoc!=-1){
					String s = team.substring(streakLoc + WON.length()).trim();
					streak = Double.parseDouble(s);
				}
				else{
					streakLoc = team.indexOf(LOST);
					if(streakLoc!=-1){
						String s = team.substring(streakLoc + LOST.length()).trim();
						streak = 0.0 - Double.parseDouble(s);
					}					
				}
				d.addDatum(new Datum(streak, Double.parseDouble(percentage), name, 0, null));
			}
		}
		return d;
	}

	final public static long SIX_HOURS = 1000 * 60 * 60 * 6;
	public Enumeration getDatasets(){
		Date d = new Date();
		if((lastRetrieved==null)||(d.getTime()-lastRetrieved.getTime()>SIX_HOURS)){
			lastRetrieved = new Date();
			generate();
		}
		return datasets.elements();
	}
	
	public String getUniqueIdentifier(){
		Date d = new Date();
		if((lastRetrieved==null)||(d.getTime()-lastRetrieved.getTime()>SIX_HOURS)){
			lastRetrieved = new Date();
			generate();
		}
		return lastRetrieved.toString();
	}
	
	public DataProvider getOneTeamDataProvider(String teamName){
		if(teamName.equals("CHI WHITE SOX"))
			teamName = "CHICAGO WSOX";
		if(teamName.equals("ST LOUIS"))
			teamName = "ST. LOUIS";
		return new OneTeamDataProvider(teamName, this);
	}
	
	public String[] getTeamNames(){
		Enumeration en = getDatasets();
		ArrayList list = new ArrayList();
		while(en.hasMoreElements()){
			list.add(((Dataset)en.nextElement()).getName());
		}
		String[] names = new String[list.size()];
		Object[] objs = list.toArray();
		for(int i=0;i<names.length;i++)
			names[i] = (String)objs[i];
		return names;
	}
	
	class OneTeamDataProvider implements DataProvider {
		String teamName = null;
		ALSchedule schedule;
		
		OneTeamDataProvider(String teamName, ALSchedule schedule){
			this.teamName = teamName;
			this.schedule = schedule;
		}
		
		public Enumeration getDatasets(){
			Enumeration sets = schedule.getDatasets();
			while(sets.hasMoreElements()){
				Dataset d = (Dataset)sets.nextElement();
				if(d.getName().equals(teamName)){
					ArrayList list = new ArrayList();
					list.add(d);
					return Collections.enumeration(list);
				}
			}
			return null;
		}
		
		public String getUniqueIdentifier(){
			return teamName + schedule.getUniqueIdentifier();
		}
	}

	//for unit testing - make sure the site is alive and retrieves values for us
	public static void main(String[] args){
		ALSchedule data = new ALSchedule();
		data.generate();
		
		/*
		for(int i=0;i<data.datasets.size();i++){
			System.out.println(data.getDatasetLinks(i));
		}
		*/
		//Enumeration e = data.getDatasets();
		
		/*
		while(e.hasMoreElements()){
			Dataset d = (Dataset)e.nextElement();
			System.out.println("got " + d.getName());
			System.out.println("with " + d.getData().size() + " elements");
			for(int i=0;i<d.getData().size();i++){
				System.out.println("dat:" + d.getDataElementAt(i).getLabel() + ", " + d.getDataElementAt(i).getX() + ", " + d.getDataElementAt(i).getY());
			}
		}
		*/
	}
}
