package examples;

import java.net.*;
import java.util.*;
import java.io.*;
import java.text.*;

import com.ve.kavachart.utility.DataProvider;
import com.ve.kavachart.chart.Dataset;

import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
/**
 * A "screen scraper" to grab text data from the US Department of Labor web site
 */
public class BLSData extends JPanel implements DataProvider, ActionListener {

	boolean noisy = false;
	Vector datasets = null;
	String seriesID = "LAUSM74000006"; //id corresponding with San Jose
	
	int startYear, endYear;
	public boolean doAll = false;
	
	Dataset d_laborForce, d_employment, d_unemployment, d_employmentRate;
	Color[] initialColors = {
		new Color(0x6666fa),
		new Color(0xffff33),
		new Color(0x99ff00),
		new Color(0x9999ff),
		new Color(0xff6666),
		new Color(0x99ffff),
		new Color(0xff9900)
	};
	
	public BLSData(){
		initUI();
	}

	/**
	 * DataProvider interface implementation.  Gets our data.
	 */
	public Enumeration getDatasets(){
		if(datasets==null)
			generate();
		datasets = new Vector();
		if(laborForceCheckBox.isSelected()||doAll)
			datasets.addElement(d_laborForce);
		if(employmentCheckBox.isSelected()||doAll)
			datasets.addElement(d_employment);
		if(unemploymentCheckBox.isSelected()||doAll)
			datasets.addElement(d_unemployment);
		if(unemploymentRateCheckBox.isSelected()||doAll)
			datasets.addElement(d_employmentRate);
		if(datasets.size()==0){
			datasets.addElement(d_laborForce);
			laborForceCheckBox.setSelected(true);
		}
		return datasets.elements();
	}

	/**
	 * DataProvider interface implementation.  Gets an identifier for this data.
	 */
	public String getUniqueIdentifier(){
		return "id:" + seriesID + startYear + endYear;
	}

	public void setFromYear(int year){
		startYear = year;
		datasets = null;
	}

	public void setToYear(int year){
		endYear = year;
		datasets = null;
	}

	public void setSeriesID(String id){
		seriesID = id;
		datasets = null;
	}

	/**
	 * This method does the work of retrieving the data from the BLS web site and converting it into
	 * KavaChart Dataset classes.  It's pretty simple, really.  Just a bunch of stuff to generate
	 * and parse the page.
	 */
	private void generate(){
		Vector strings = new Vector();
		try{
		String blsServletString = "http://data.bls.gov/servlet/SurveyOutputServlet?";
		StringBuffer sb = new StringBuffer(blsServletString);
		sb.append("output_type=default&years_option=specific_years&");
		sb.append("from_year=" + startYear + "&to_year=" + endYear);
		sb.append("&output_view=data&periods_option=all_periods&output_format=text&delimiter=comma&reformat=true&request_action=get_data&initial_request=true&data_tool=surveymost&");
		sb.append("series_id=" + seriesID);
		URL blsURL = new URL(sb.toString());
		BufferedReader reader = new BufferedReader(new InputStreamReader(blsURL.openStream()));
		String line = reader.readLine(); 
		while(!line.startsWith("<PRE")){ //strip off header
			line = reader.readLine();
		}
		for(int i=0;i<10;i++)
			reader.readLine(); //strip off ten more lines


		line = reader.readLine();
		while(line.length()>2){ //read interesting stuff
			strings.addElement(line);
			line = reader.readLine();
		}
		}
		catch(IOException io){
			return;
		}

		SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMM");
		//now convert the strings into Dataset classes
		Enumeration en = strings.elements();
		String datasetNames = (String)en.nextElement(); //throwaway

		d_laborForce = new Dataset();
		d_laborForce.setName("Labor Force Size");
		d_employment = new Dataset();
		d_employment.setName("Total Employment");
		d_unemployment = new Dataset();
		d_unemployment.setName("Total Unemployment");
		d_employmentRate = new Dataset();
		d_employmentRate.setName("Unemployment Rate");
		while(en.hasMoreElements()){
			String s = (String)en.nextElement();
			StringTokenizer st = new StringTokenizer(s, ",");
			String year = st.nextToken();
			String month = st.nextToken();
			if(!month.equals("Annual")){
				String laborForce = st.nextToken();
				String employment = st.nextToken();
				String unemployment = st.nextToken();
				String unemploymentRate = st.nextToken();

				Date d = null;
				try{
					d = dateFormat.parse(year + month);
				}catch(ParseException p){
					d = new Date();
				}

				double time = (double)d.getTime();
				d_laborForce.addPoint(time, parseDouble(laborForce), null);
				d_employment.addPoint(time, parseDouble(employment), null);
				d_unemployment.addPoint(time, parseDouble(unemployment), null);
				d_employmentRate.addPoint(time, parseDouble(unemploymentRate)/100., null);
			}
		}
		
		d_laborForce.getGc().setFillColor(initialColors[0]);
		d_employment.getGc().setFillColor(initialColors[1]);
		d_unemployment.getGc().setFillColor(initialColors[2]);
		d_employmentRate.getGc().setFillColor(initialColors[3]);

		d_laborForce.getGc().setLineColor(initialColors[0]);
		d_employment.getGc().setLineColor(initialColors[1]);
		d_unemployment.getGc().setLineColor(initialColors[2]);
		d_employmentRate.getGc().setLineColor(initialColors[3]);

	}
	
	private double parseDouble(String in){
		try{
			return Double.parseDouble(in);
		}catch(Exception e){
			in = in.substring(0, in.indexOf("("));
			return Double.parseDouble(in);
		}
	}

	//for unit testing - make sure the site is alive and retrieves values for us
	public static void main(String[] args){
		BLSData data = new BLSData();
		JFrame f = new JFrame();
		f.getContentPane().add(data);
		f.setSize(500, 500);
		f.setVisible(true);
		/*
		Enumeration e = data.getDatasets();
		while(e.hasMoreElements()){
			Dataset d = (Dataset)e.nextElement();
			System.out.println("got " + d.getName());
			System.out.println("with " + d.getData().size() + " elements");
		}
		*/
	}
	
	JCheckBox laborForceCheckBox, employmentCheckBox, unemploymentCheckBox, unemploymentRateCheckBox;
	JComboBox startYearComboBox, endYearComboBox;
	
	private void initUI(){
		laborForceCheckBox = new JCheckBox("Labor Force Size");
		employmentCheckBox = new JCheckBox("Total Employment");
		unemploymentCheckBox = new JCheckBox("Total Unemployment");
		unemploymentRateCheckBox = new JCheckBox("Unemployment Rate");
		
		GridBagLayout gbl = new GridBagLayout();
		setLayout(gbl);
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.WEST;
		
		gbc.insets.top = 20;
		gbc.gridy = 0;

		gbc.gridwidth = 2;
		JLabel title = new JLabel("San Jose Labor Statistics");
		add(title);
		
		JLabel startYearLabel = new JLabel("Start Year");
		JLabel endYearLabel = new JLabel("End Year");
		Date now = new Date();
		Calendar c = Calendar.getInstance();
		c.setTime(now);
		int thisYear = c.get(Calendar.YEAR);
		String[] years = new String[thisYear - 1992];
		for(int i=0;i<=thisYear - 1993;i++)
			years[i] = Integer.toString(i + 1993);
			
		startYearComboBox = new JComboBox(years);
		startYearComboBox.setSelectedItem(Integer.toString(thisYear-4));
		endYearComboBox = new JComboBox(years);
		endYearComboBox.setSelectedItem(Integer.toString(thisYear));
		
		gbc.gridwidth = 1;
		gbc.gridy++;
		add(startYearLabel);
		gbl.setConstraints(startYearLabel, gbc);
		gbc.gridx = 1;
		
		add(startYearComboBox);
		gbl.setConstraints(startYearComboBox, gbc);
		
		gbc.gridx = 0;
		gbc.gridy++;
		add(endYearLabel);
		gbl.setConstraints(endYearLabel, gbc);
		
		gbc.gridx = 1;
		add(endYearComboBox);
		gbl.setConstraints(endYearComboBox, gbc);
		
		gbc.gridwidth = 2;
		gbc.gridx = 0;
		gbc.gridy++;
		
		add(laborForceCheckBox);
		gbl.setConstraints(laborForceCheckBox, gbc);
		
		gbc.gridy++;
		add(employmentCheckBox);
		gbl.setConstraints(employmentCheckBox, gbc);
		
		gbc.gridy++;
		add(unemploymentCheckBox);
		gbl.setConstraints(unemploymentCheckBox, gbc);
		
		gbc.gridy++;
		add(unemploymentRateCheckBox);
		gbl.setConstraints(unemploymentRateCheckBox, gbc);
		
		
		laborForceCheckBox.setSelected(true);
		
		laborForceCheckBox.addActionListener(this);
		employmentCheckBox.addActionListener(this);
		unemploymentCheckBox.addActionListener(this);
		unemploymentRateCheckBox.addActionListener(this);
		startYearComboBox.addActionListener(this);
		endYearComboBox.addActionListener(this);
		setFromYear(Integer.parseInt((String)startYearComboBox.getSelectedItem()));
		setToYear(Integer.parseInt((String)endYearComboBox.getSelectedItem()));
		
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent arg0) {
		int s = Integer.parseInt((String)startYearComboBox.getSelectedItem());
		if(s!=startYear)
			setFromYear(s);
		int e = Integer.parseInt((String)endYearComboBox.getSelectedItem());
		if(e!=endYear)
			setToYear(e);
		firePropertyChange(null, null, null);
	}
}

