/*
* Copyright 1997-2003 Visual Engineering, Inc. All Rights Reserved.
*
* This software is the proprietary information of Visual Engineering, Inc.
* Use is subject to license terms.
*/
package examples;

import java.net.*;
import java.util.*;
import java.io.*;

import com.ve.kavachart.utility.*;
import com.ve.kavachart.chart.*;

public class BayAreaWeather implements DataProvider {

	Vector datasets = new Vector();
	boolean noisy = false;
	Date lastChecked = null;
	ArrayList cityNames, temperatures;

	public void generate(){
		//System.out.println("getting new bay area weather info");
		lastChecked = new Date();
		datasets.removeAllElements();
		cityNames = new ArrayList();
		temperatures = new ArrayList();
		String entireContent = null;
		try{
		//URL noaaURL = new URL("http://newweb.wrh.noaa.gov/mtr/gettext.php?pil=RWR&sid=CA");
		URL noaaURL = new URL("http://newweb.wrh.noaa.gov/mtr/getrwr.php?sid=ca&node=sac");
		BufferedReader reader = new BufferedReader(new InputStreamReader(noaaURL.openStream()));
		String line = reader.readLine();
		entireContent = line;
		while(line!=null){
			line = reader.readLine();
			entireContent = entireContent + line;
		}
		reader.close();
		}
		catch(IOException io){
			return;
		}

		String[] contentStrings = entireContent.split("<br>");
		int lineNumber = 0;
		while(contentStrings[lineNumber].indexOf("BAY AREA")==-1){ //ignore top level summary
			lineNumber++;
		}
		lineNumber++; //strip off one more line
		lineNumber++; //strip off one more line
		lineNumber++;

		while((lineNumber<contentStrings.length)&&(contentStrings[lineNumber].indexOf("<hr>")==-1)){ //read interesting stuff
			String cityName = contentStrings[lineNumber].substring(0, 15).trim();
			String temperature = contentStrings[lineNumber].substring(24, 28).trim();
			if((!temperature.equals("MM"))&&(temperature.indexOf("A")==-1)&&(temperature.length()>0)){ //temp is available...
				temperatures.add(temperature);
				cityNames.add(cityName);
				//System.out.println("temp:" + temperature + " city:" + cityName);
			}
			lineNumber++;
		}

		Dataset dataset = new Dataset();
		dataset.getGc().setFillColor(java.awt.Color.GREEN);
		dataset.getGc().setLineColor(java.awt.Color.GREEN);
		dataset.setName("Bay Area Temperatures:" + lastChecked);
		for(int i=0;i<temperatures.size();i++){
			String temp = (String)temperatures.get(i);
			String city = (String)cityNames.get(i);
			dataset.addPoint(i, Double.parseDouble(temp), city);
			//Datum dat = new Datum(Double.valueOf(temp).doubleValue(), city, i, null);
			//dataset.addDatum(dat);
		}
		datasets.addElement(dataset);
	}

	static private long HOUR = 1000 * 60 * 60;
	public Enumeration getDatasets(){
		Date now = new Date();
		if((datasets.size()==0)||(now.getTime()-lastChecked.getTime()>HOUR))
			generate();
		return datasets.elements();
	}

	public String getUniqueIdentifier(){
		if(lastChecked==null)
			generate();
		return Long.toString(lastChecked.getTime());
	}

	//for unit testing - make sure the site is alive and retrieves values for us
	public static void main(String[] args){
		BayAreaWeather data = new BayAreaWeather();
		Enumeration e = data.getDatasets();
		Dataset d = (Dataset)e.nextElement();
		for(int i=0;i<d.getData().size();i++){
			System.out.println(i + ":" + d.getDataElementAt(i).getLabel() + " - " + d.getDataElementAt(i).getY());
		}
	}
}
