/*
* Copyright 1997-2004 Visual Engineering, Inc. All Rights Reserved.
*
* This software is the proprietary information of Visual Engineering, Inc.
* Use is subject to license terms.
*/
package examples;

import java.net.*;
import java.util.*;
import java.io.*;


public class BayAreaWeatherWebService {

	boolean noisy = false;
	long lastChecked = System.currentTimeMillis();
	ArrayList cityNames, temperatures;


	public void generate(){
		lastChecked = System.currentTimeMillis();
		//System.out.println("getting new bay area weather info");
		cityNames = new ArrayList();
		temperatures = new ArrayList();
		String entireContent = null;
		try{
		//URL noaaURL = new URL("http://newweb.wrh.noaa.gov/mtr/gettext.php?pil=RWR&sid=CA");
		URL noaaURL = new URL("http://newweb.wrh.noaa.gov/mtr/getrwr.php?sid=ca&node=sac");
		BufferedReader reader = new BufferedReader(new InputStreamReader(noaaURL.openStream()));
		String line = reader.readLine();
		entireContent = line;
		while(line!=null){
			line = reader.readLine();
			entireContent = entireContent + line;
		}
		reader.close();
		}
		catch(IOException io){
			return;
		}

		String[] contentStrings = entireContent.split("<br>");
		int lineNumber = 0;
		while(contentStrings[lineNumber].indexOf("BAY AREA")==-1){ //ignore top level summary
			lineNumber++;
		}
		lineNumber++; //strip off one more line
		lineNumber++; //strip off one more line
		lineNumber++;

		while((lineNumber<contentStrings.length)&&(contentStrings[lineNumber].indexOf("<hr>")==-1)){ //read interesting stuff
			String cityName = contentStrings[lineNumber].substring(0, 15).trim();
			String temperature = contentStrings[lineNumber].substring(24, 28).trim();
			if((!temperature.equals("MM"))&&(temperature.indexOf("A")==-1)&&(temperature.length()>0)){ //temp is available...
				temperatures.add(temperature);
				cityNames.add(cityName);
				//System.out.println("temp:" + temperature + " city:" + cityName);
			}
			lineNumber++;
		}
	}

	public void doWeather(Writer out){
		if(cityNames==null)
			generate();
		if(System.currentTimeMillis() - lastChecked>HOUR)
			generate();
			
		String[] cities = new String[cityNames.size()];
		cityNames.toArray(cities);
		String[] temps = new String[temperatures.size()];
		temperatures.toArray(temps);
		
		try{
		out.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\" ?>\n");
		out.write("<root>\n");
		out.write("\t<dataset name=\"Bay Area Weather\">\n");
		for(int i=0;i<cities.length;i++){
			try{
			int deg = Integer.parseInt(temps[i]);
			out.write("\t\t<datum label=\"" + cities[i] + "\" y=\"" + temps[i] + "\" />\n");
			}catch(Exception throwOutThisInvalidTemp){
			}
		}
		out.write("\t</dataset>\n");
		out.write("</root>\n");
		}catch(Exception ignored){
		}

	}
	private long HOUR = 1000 * 60 * 60;

	//for unit testing - make sure the site is alive and retrieves values for us
	public static void main(String[] args){
		BayAreaWeatherWebService data = new BayAreaWeatherWebService();
		PrintWriter pw = new PrintWriter(System.out);
		data.doWeather(pw);
		pw.flush();
	}
}
