package examples;

import java.net.*;
import java.util.*;
import java.io.*;

import com.ve.kavachart.utility.DataProvider;
import com.ve.kavachart.chart.Dataset;
import com.ve.kavachart.chart.Datum;

/**
 * A screen scraper class to get NBA standing data from USAToday
 */
public class MLBSectormapData implements DataProvider {

	String startTag = "<!start_tag>";
	String endTag = "<!end_tag>";
	public boolean noisy = false;
	Vector datasets = null;
	Date lastRetrieved = null;

	/**
	 * This method does the work of retrieving the data from the BLS web site and converting it into
	 * KavaChart Dataset classes.  It's pretty simple, really.  Just a bunch of stuff to generate
	 * and parse the page.
	 */
	private void generate(){
		datasets = new Vector();
		Vector nationalLeagueEastStrings = new Vector();
		Vector nationalLeagueCentralStrings = new Vector();
		Vector nationalLeagueWestStrings = new Vector();
		Vector pacificDivisionStrings = new Vector();
		try{
		String nbaURLString = "http://www.usatoday.com/sports/baseball/nl/standings.htm";
		URL nbaURL = new URL(nbaURLString);
		BufferedReader reader = new BufferedReader(new InputStreamReader(nbaURL.openStream()));
		String line = reader.readLine(); 
		while(!line.startsWith(startTag)){ //strip off header
			line = reader.readLine();
		}
		for(int i=0;i<5;i++)
			reader.readLine();
		//atlantic div
		for(int i=0;i<5;i++)
			nationalLeagueEastStrings.addElement(reader.readLine());
		//header stuff
		for(int i=0;i<8;i++)
			reader.readLine();
		//central div
		for(int i=0;i<6;i++)
			nationalLeagueCentralStrings.addElement(reader.readLine());
		//header stuff
		for(int i=0;i<8;i++)
			reader.readLine();
		for(int i=0;i<5;i++)
			nationalLeagueWestStrings.addElement(reader.readLine());
		}
		catch(IOException io){
			return;
		}
		datasets.addElement(buildDataset("NL East", nationalLeagueEastStrings));
		datasets.addElement(buildDataset("NL Central", nationalLeagueCentralStrings));
		datasets.addElement(buildDataset("NL West", nationalLeagueWestStrings));

	}
	
	final static String WON = "WON";
	final static String LOST = "LOST";

	private Dataset buildDataset(String division, Vector teams){
		Dataset d = new Dataset();
		d.setName(division);
		Enumeration en = teams.elements();
		while(en.hasMoreElements()){
			String team = (String)en.nextElement();
			//System.out.println(team + ", " + team.length());
			if(team.length()>1){
				String name = team.substring(0, 16).trim();
				team = team.substring(17);
				StringTokenizer st = new StringTokenizer(team);
				String wins = st.nextToken();
				String losses = st.nextToken();
				String percentage = st.nextToken();
				double streak = 0.0;
				int streakLoc = team.indexOf(WON);
				if(streakLoc!=-1){
					String s = team.substring(streakLoc + WON.length()).trim();
					streak = Double.parseDouble(s);
				}
				else{
					streakLoc = team.indexOf(LOST);
					if(streakLoc!=-1){
						String s = team.substring(streakLoc + LOST.length()).trim();
						streak = 0.0 - Double.parseDouble(s);
					}					
				}
				d.addDatum(new Datum(streak, Double.parseDouble(percentage), name, 0, null));
			}
		}
		return d;
	}

	final public static long SIX_HOURS = 1000 * 60 * 60 * 6;
	public Enumeration getDatasets(){
		Date d = new Date();
		if((lastRetrieved==null)||(d.getTime()-lastRetrieved.getTime()>SIX_HOURS)){
			lastRetrieved = new Date();
			generate();
		}
		return datasets.elements();
	}
	public String getUniqueIdentifier(){
		Date d = new Date();
		if((lastRetrieved==null)||(d.getTime()-lastRetrieved.getTime()>SIX_HOURS)){
			lastRetrieved = new Date();
			generate();
		}
		return lastRetrieved.toString();
	}

	//for unit testing - make sure the site is alive and retrieves values for us
	public static void main(String[] args){
		MLBSectormapData data = new MLBSectormapData();
		data.generate();
		Enumeration e = data.getDatasets();
		while(e.hasMoreElements()){
			Dataset d = (Dataset)e.nextElement();
			System.out.println("got " + d.getName());
			System.out.println("with " + d.getData().size() + " elements");
			for(int i=0;i<d.getData().size();i++){
				System.out.println("dat:" + d.getDataElementAt(i).getLabel() + ", " + d.getDataElementAt(i).getX() + ", " + d.getDataElementAt(i).getY());
			}
		}
	}
}
