/*
* Copyright 1997-2004 Visual Engineering, Inc. All Rights Reserved.
*
* This software is the proprietary information of Visual Engineering, Inc.
* Use is subject to license terms.
*/
package examples;

import java.net.*;
import java.util.*;
import java.io.*;

import com.ve.kavachart.chart.*;
import com.ve.kavachart.utility.*;

public class MontereyBouyData implements DataProvider {

	Vector datasets = new Vector();
	boolean noisy = false;
	Date lastChecked = null;


	public void generate(){

		System.out.println("reading data from monterey bouy");

		datasets.removeAllElements();
		Vector fileLines = new Vector();
		lastChecked = new Date();

		try{
			URL noaaURL = new URL("http://www.ndbc.noaa.gov/data/realtime2/46042.txt");
			BufferedReader reader = new BufferedReader(new InputStreamReader(noaaURL.openStream()));
			String line = reader.readLine(); //strip off the titles...
			line = reader.readLine();
			while(line!=null){
				fileLines.addElement(line);
				line = reader.readLine();
			}
			reader.close();
		}
		catch(Exception e){
		}

		Vector vWindGust = new Vector();
		Vector vWindSpeed = new Vector();
		Vector vTimeStamps = new Vector();
		Vector vWaveHeight = new Vector();
		Vector vWaterTemp = new Vector();
		Vector vAirTemp = new Vector();

		int hoursOfData = fileLines.size();
		java.text.SimpleDateFormat parser = (java.text.SimpleDateFormat)java.text.DateFormat.getInstance();
		parser.applyPattern("yyyyMMddHH");

		Double discontinuity = new Double(Double.NEGATIVE_INFINITY);
		for(int i=hoursOfData-1;i>-1;i--){
			String thisLine = (String)fileLines.elementAt(i);
			StringTokenizer st = new StringTokenizer(thisLine);
			String ts = st.nextToken() + st.nextToken() + st.nextToken() + st.nextToken();
			try{
			vTimeStamps.addElement(parser.parse(ts));
			}catch(Exception bah){
				System.out.println("bad timestamp");
			}
			st.nextToken();
			st.nextToken();

			try{
				Double windSpeed = new Double(((Double.valueOf(st.nextToken()).doubleValue() * 60 * 60) / 1000) * 0.6214);
				vWindSpeed.addElement(windSpeed);
				//vWindSpeed.addElement(Double.valueOf(st.nextToken()));
			}catch(Exception e){
				vWindSpeed.addElement(discontinuity);
			}

			try{
				Double windGust = new Double(((Double.valueOf(st.nextToken()).doubleValue() * 60 * 60) / 1000) * 0.6214);
				vWindGust.addElement(windGust);
				//vWindGust.addElement(Double.valueOf(st.nextToken()));
			}catch(Exception e){
				vWindGust.addElement(discontinuity);
			}

			try{
				Double waveHt = new Double(((Double.valueOf(st.nextToken()).doubleValue() * 100)/2.54)/12);
				vWaveHeight.addElement(waveHt);
				//vWaveHeight.addElement(Double.valueOf(st.nextToken()));
			}catch(Exception e){
				vWaveHeight.addElement(discontinuity);
			}

			//skip some stuff...
			st.nextToken();st.nextToken();st.nextToken();st.nextToken();
			try{
				Double airTemp = new Double((Double.valueOf(st.nextToken()).doubleValue() * 9/5 ) + 32.);
				vAirTemp.addElement(airTemp);
			}catch(Exception e){
				vAirTemp.addElement(discontinuity);
			}

			try{
				Double waterTemp = new Double((Double.valueOf(st.nextToken()).doubleValue() * 9/5 ) + 32.);
				vWaterTemp.addElement(waterTemp);
			}catch(Exception e){
				vWaterTemp.addElement(discontinuity);
			}

		}

		//now turn all this data into KavaChart Dataset classes and add the classes to our Vector "datasets"
		Dataset windGust = new Dataset();
		windGust.setName("Wind Gust");
		populateDataset(windGust, vTimeStamps, vWindGust);
		datasets.addElement(windGust);

		Dataset windSpeed = new Dataset();
		windSpeed.setName("Wind Speed");
		populateDataset(windSpeed, vTimeStamps, vWindSpeed);
		datasets.addElement(windSpeed);

		Dataset waveHeight = new Dataset();
		waveHeight.setName("Wave Height");
		populateDataset(waveHeight, vTimeStamps, vWaveHeight);
		datasets.addElement(waveHeight);

		Dataset waterTemp = new Dataset();
		waterTemp.setName("Water Temp");
		populateDataset(waterTemp, vTimeStamps, vWaterTemp);
		datasets.addElement(waterTemp);

		Dataset airTemp = new Dataset();
		airTemp.setName("Air Temp");
		populateDataset(airTemp, vTimeStamps, vAirTemp);
		datasets.addElement(airTemp);

	}

	private void populateDataset(Dataset d, Vector times, Vector vals){
		for(int i=0;i<times.size();i++){
			Date date = (Date)times.elementAt(i);
			Double val = (Double)vals.elementAt(i);
			d.addPoint((double)date.getTime(), val.doubleValue(), null);
		}
	}

	static private long HOUR = 1000 * 60 * 60;
	public Enumeration getDatasets(){
		Date now = new Date();
		if((datasets.size()==0)||(now.getTime()-lastChecked.getTime()>HOUR))
			generate();
		return datasets.elements();
	}

	public String getUniqueIdentifier(){
		if(lastChecked==null)
			generate();
		return Long.toString(lastChecked.getTime());
	}

	//for unit testing - make sure the site is alive and retrieves values for us
	public static void main(String[] args){
		MontereyBouyData data = new MontereyBouyData();
		Enumeration e = data.getDatasets();
		while(e.hasMoreElements()){
		Dataset d = (Dataset)e.nextElement();
		System.out.println(d.getName() + "---------------------------");
		for(int i=d.getData().size()-10;i<d.getData().size();i++){
			System.out.println(i + ":" + new Date((long)d.getDataElementAt(i).getX()) + " - " + d.getDataElementAt(i).getY());
		}
		}
	}
}