/*
* Copyright 1997-2004 Visual Engineering, Inc. All Rights Reserved.
*
* This software is the proprietary information of Visual Engineering, Inc.
* Use is subject to license terms.
*/
package examples;

import java.net.*;
import java.util.*;
import java.io.*;

public class MontereyBuoyDataWebService {

	boolean noisy = false;
	long lastChecked = System.currentTimeMillis();

	ArrayList vWindGust = null;
	ArrayList vWindSpeed = null;
	ArrayList vTimeStamps = null;
	ArrayList vWaveHeight = null;
	ArrayList vWaterTemp = null;
	ArrayList vAirTemp = null;

	public void generate(){

		Vector fileLines = new Vector();
		lastChecked = System.currentTimeMillis();

		try{
			URL noaaURL = new URL("http://www.ndbc.noaa.gov/data/realtime2/46042.txt");
			BufferedReader reader = new BufferedReader(new InputStreamReader(noaaURL.openStream()));
			String line = reader.readLine(); //strip off the titles...
			line = reader.readLine();
			while(line!=null){
				fileLines.addElement(line);
				line = reader.readLine();
			}
			reader.close();
		}
		catch(Exception e){
		}

		vWindGust = new ArrayList();
		vWindSpeed = new ArrayList();
		vTimeStamps = new ArrayList();
		vWaveHeight = new ArrayList();
		vWaterTemp = new ArrayList();
		vAirTemp = new ArrayList();

		int hoursOfData = fileLines.size();

		Double discontinuity = new Double(Double.NEGATIVE_INFINITY);
		for(int i=hoursOfData-1;i>-1;i--){
			String thisLine = (String)fileLines.elementAt(i);
			StringTokenizer st = new StringTokenizer(thisLine);
			String ts = st.nextToken() + st.nextToken() + st.nextToken() + st.nextToken();
			vTimeStamps.add(ts);

			st.nextToken();

			vWindSpeed.add(st.nextToken());
			vWindGust.add(st.nextToken());
			vWaveHeight.add(st.nextToken());
			
			//skip some stuff...
			st.nextToken();st.nextToken();st.nextToken();st.nextToken();
			
			vAirTemp.add(st.nextToken());			
			vWaterTemp.add(st.nextToken());
		}

	}


	static private long HOUR = 1000 * 60 * 60;
	public void getInfo(java.io.Writer out){
		Date now = new Date();
		if(vTimeStamps==null||System.currentTimeMillis()-lastChecked>HOUR)
			generate();
		
		try{
		out.write("<root dateFormat=\"yyyyMMddHH\">\n");
		
		//now turn all this data into KavaChart Dataset classes and add the classes to our Vector "datasets"
		createDataset(out, "Wind Gust", vWindGust);
		createDataset(out, "Wind Speed", vWindSpeed);
		createDataset(out, "Wave Height", vWaveHeight);
		createDataset(out, "Water Temp", vWaterTemp);
		createDataset(out, "Air Temp", vAirTemp);
		out.write("</root>\n");
		}catch(Exception ignored){
		}
	}
	
	private void createDataset(Writer out, String name, ArrayList values){
		try{
			out.write("\t<dataset name=\"" + name + "\">\n");
			Object[] vals = values.toArray();
			Object[] times = vTimeStamps.toArray();
			for(int i=0;i<vals.length;i++){
				out.write("\t\t<datum date=\"" + times[i] + "\" y=\"" + vals[i] + "\" />\n");
			}
			out.write("\t</dataset>\n");
		}catch(Exception ignored){
		}
	}

}