package examples;

import java.net.*;
import java.util.*;
import java.io.*;

import com.ve.kavachart.utility.DataProvider;
import com.ve.kavachart.chart.*;

/**
 * A screen scraper class to get NBA standing data from USAToday
 */
public class NFLData implements DataProvider {

	String startTag = "<PRE>";
	String endTag = "</PRE>";
	public boolean noisy = false;
	ArrayList datasets = null;
	Date lastRetrieved = null;

	/**
	 * This method does the work of retrieving the data from the BLS web site and converting it into
	 * KavaChart Dataset classes.  It's pretty simple, really.  Just a bunch of stuff to generate
	 * and parse the page.
	 */
	private void generate(){
		datasets = new ArrayList();
		ArrayList afcEast = new ArrayList();
		ArrayList afcNorth = new ArrayList();
		ArrayList afcSouth = new ArrayList();
		ArrayList afcWest = new ArrayList();

		ArrayList nfcEast = new ArrayList();
		ArrayList nfcNorth = new ArrayList();
		ArrayList nfcSouth = new ArrayList();
		ArrayList nfcWest = new ArrayList();

		try{
		String nflURLString = "http://football.about.com/library/weekly/bl_nflstandings2004.htm";
		URL nflURL = new URL(nflURLString);
		BufferedReader reader = new BufferedReader(new InputStreamReader(nflURL.openStream()));
		String line = reader.readLine(); 
		while(!line.startsWith(startTag)){ //strip off header
			line = reader.readLine();
		}
		line = reader.readLine();
		while(!line.startsWith(endTag)){
			if(line.startsWith("AFC East")){
				line = reader.readLine();
				while(line.length()>14){
					afcEast.add(line);
					line = reader.readLine();
				}
			}
			else if(line.startsWith("AFC West")){
				line = reader.readLine();
				while(line.length()>14){
					afcWest.add(line);
					line = reader.readLine();
				}
			}
			else if(line.startsWith("AFC North")){
				line = reader.readLine();
				while(line.length()>14){
					afcNorth.add(line);
					line = reader.readLine();
				}
			}
			else if(line.startsWith("AFC South")){
				line = reader.readLine();
				while(line.length()>14){
					afcSouth.add(line);
					line = reader.readLine();
				}
			}
			
			if(line.startsWith("NFC East")){
				line = reader.readLine();
				while(line.length()>14){
					nfcEast.add(line);
					line = reader.readLine();
				}
			}
			else if(line.startsWith("NFC West")){
				line = reader.readLine();
				while(line.length()>14){
					nfcWest.add(line);
					line = reader.readLine();
				}
			}
			else if(line.startsWith("NFC North")){
				line = reader.readLine();
				while(line.length()>14){
					nfcNorth.add(line);
					line = reader.readLine();
				}
			}
			else if(line.startsWith("NFC South")){
				line = reader.readLine();
				while(line.length()>14){
					nfcSouth.add(line);
					line = reader.readLine();
				}
			}
			line = reader.readLine();
		}
		}
		catch(IOException io){
			return;
		}
		datasets.add(buildDataset("AFC West", afcWest));
		datasets.add(buildDataset("AFC East", afcEast));
		datasets.add(buildDataset("AFC North", afcNorth));
		datasets.add(buildDataset("AFC South", afcSouth));

		datasets.add(buildDataset("NFC West", nfcWest));
		datasets.add(buildDataset("NFC East", nfcEast));
		datasets.add(buildDataset("NFC North", nfcNorth));
		datasets.add(buildDataset("NFC South", nfcSouth));

	}

	private Dataset buildDataset(String division, ArrayList teams){
		//System.out.println("-----");
		Dataset d = new Dataset();
		d.setName(division);
		Enumeration en = Collections.enumeration(teams);
		while(en.hasMoreElements()){
			String team = (String)en.nextElement();
			//System.out.println(team + ", " + team.length());
			if(team.length()>15){
				String name = team.substring(0, 14).trim();
				team = team.substring(14).trim();
				StringTokenizer st = new StringTokenizer(team);
				String wins = st.nextToken();
				String losses = st.nextToken();
				String percentage = st.nextToken();
				//d.addDatum(new Datum(0.0, Double.parseDouble(percentage), name, 0, null));
				Datum dat = new Datum(0.0, Double.parseDouble(wins), name, 0, null);
				dat.setY2(Double.parseDouble(losses));
				d.addDatum(dat);
			}
		}
		return d;
	}

	final public static long SIX_HOURS = 1000 * 60 * 60 * 6;
	public Enumeration getDatasets(){
		Date d = new Date();
		if((lastRetrieved==null)||(d.getTime()-lastRetrieved.getTime()>SIX_HOURS)){
			lastRetrieved = new Date();
			generate();
		}
		return Collections.enumeration(datasets);
	}
	public String getUniqueIdentifier(){
		Date d = new Date();
		if((lastRetrieved==null)||(d.getTime()-lastRetrieved.getTime()>SIX_HOURS)){
			lastRetrieved = new Date();
			generate();
		}
		return lastRetrieved.toString();
	}

	//for unit testing - make sure the site is alive and retrieves values for us
	public static void main(String[] args){
		NFLData data = new NFLData();
		data.generate();
		Enumeration e = data.getDatasets();
		while(e.hasMoreElements()){
			Dataset d = (Dataset)e.nextElement();
			System.out.println("got " + d.getName());
			System.out.println("with " + d.getData().size() + " elements");
		}
	}
}
