package examples;

import java.net.*;
import java.util.*;
import java.io.*;

import com.ve.kavachart.utility.DataProvider;
import com.ve.kavachart.chart.Dataset;
import com.ve.kavachart.chart.Datum;

/**
 * A screen scraper class to get NBA standing data from USAToday
 */
public class NLALSectormapData implements DataProvider {

	boolean goLive = false;
	
	static String[] NL_EastStrings = {
		"X-ATLANTA        96   66  .593   -     49-32  47-34  51-25 18-18 19-13 LOST 2",
		"PHILADELPHIA     86   76  .531  10     42-39  44-37  39-37 18-18 20-12 WON  1",
		"FLORIDA          83   79  .512  13     42-38  41-41  43-33 15-21 18-14 LOST 3",
		"NY METS          71   91  .438  25     38-43  33-48  29-47 15-21 17-15 WON  2",
		"MONTREAL         67   95  .414  29     35-45  32-50  28-48 17-19 15-17 LOST 1",
	};
	static String[] NL_CentralStrings = {
		"X-ST LOUIS      105   57  .648   -     53-28  52-29  19-11 54-36 21-09 WON  1",
		"Y-HOUSTON        92   70  .568  13     48-33  44-37  16-14 55-35 14-16 WON  7",
		"CHICAGO CUBS     89   73  .549  16     45-37  44-36  16-14 50-40 15-15 WON  1",
		"CINCINNATI       76   86  .469  29     40-41  36-45  18-12 38-52 15-15 LOST 2",
		"PITTSBURGH       72   89  .447  32 1/2 39-41  33-48  17-13 37-52 16-14 WON  2",
		"MILWAUKEE        67   94  .416  37 1/2 36-45  31-49  11-19 35-54 13-17 LOST 1",
	};
	static String[] NL_WestStrings = {
		"X-LOS ANGELES    93   69  .574   -     49-32  44-37  14-18 22-14 47-29 LOST 1",
		"SAN FRANCISCO    91   71  .562   2     47-35  44-36  19-13 20-16 41-35 WON  1",
		"SAN DIEGO        87   75  .537   6     42-39  45-36  18-14 19-17 42-34 LOST 2",
		"COLORADO         68   94  .420  25     38-43  30-51  11-21 10-26 39-37 LOST 4",
		"ARIZONA          51  111  .315  42     29-52  22-59  09-23 15-21 21-55 WON  2",
	};
	static String[] AL_EastStrings = {
		"X-NY YANKEES    101   61  .623   -     57-24  44-37  49-27 20-12 22-14 WON  1",
		"Y-BOSTON         98   64  .605   3     55-26  43-38  48-28 19-13 22-14 LOST 1",
		"BALTIMORE        78   84  .481  23     38-43  40-41  37-39 21-15 15-17 WON  1",
		"TAMPA BAY        70   91  .435  30 1/2 41-39  29-52  26-49 19-17 10-22 WON  1",
		"TORONTO          67   94  .416  33 1/2 40-41  27-53  29-46 13-19 17-19 LOST 1",
	};
	static String[] AL_CentralStrings = {
		"X-MINNESOTA      92   70  .568   -     49-32  43-38  19-17 46-30 16-16 LOST 1",
		"CHI WHITE SOX    83   79  .512   9     46-35  37-44  16-16 40-36 19-17 WON  1",
		"CLEVELAND        80   82  .494  12     44-37  36-45  17-15 36-40 17-19 WON  1",
		"DETROIT          72   90  .444  20     38-43  34-47  12-20 36-40 15-21 LOST 1",
		"KANSAS CITY      58  104  .358  34     33-47  25-57  12-24 32-44 08-24 LOST 1",
	};
	static String[] AL_WestStrings = {
		"X-ANAHEIM        92   70  .568   -     45-36  47-34  25-18 28-15 32-26 LOST 1",
		"OAKLAND          91   71  .562   1     52-29  39-42  23-20 27-16 31-27 WON  1",
		"TEXAS            89   73  .549   3     51-30  38-43  25-18 23-20 31-27 WON  2",
		"SEATTLE          63   99  .389  29     38-44  25-55  13-30 19-24 22-36 LOST 2",
	};	String startTag = "<!start_tag>";
	String endTag = "<!end_tag>";
	public boolean noisy = false;
	Vector datasets = null;
	Date lastRetrieved = null;
	
	boolean doAL = true;
	boolean doNL = true;
	
	String[] teamLinks = {
		"http://anaheim.angels.mlb.com/NASApp/mlb/index.jsp?c_id=ana",
		"http://arizona.diamondbacks.mlb.com/NASApp/mlb/index.jsp?c_id=ari",
		"http://atlanta.braves.mlb.com/NASApp/mlb/index.jsp?c_id=atl",
		"http://baltimore.orioles.mlb.com/NASApp/mlb/index.jsp?c_id=bal",
		"http://boston.redsox.mlb.com/NASApp/mlb/index.jsp?c_id=bos",
		"http://chicago.cubs.mlb.com/NASApp/mlb/index.jsp?c_id=chc",
		"http://chicago.whitesox.mlb.com/NASApp/mlb/index.jsp?c_id=cws",
		"http://cincinnati.reds.mlb.com/NASApp/mlb/index.jsp?c_id=cin",
		"http://cleveland.indians.mlb.com/NASApp/mlb/index.jsp?c_id=cle",
		"http://colorado.rockies.mlb.com/NASApp/mlb/index.jsp?c_id=col",
		"http://detroit.tigers.mlb.com/NASApp/mlb/index.jsp?c_id=det",
		"http://florida.marlins.mlb.com/NASApp/mlb/index.jsp?c_id=fla",
		"http://houston.astros.mlb.com/NASApp/mlb/index.jsp?c_id=hou",
		"http://kansascity.royals.mlb.com/NASApp/mlb/index.jsp?c_id=kc",
		"http://losangeles.dodgers.mlb.com/NASApp/mlb/index.jsp?c_id=la",
		"http://milwaukee.brewers.mlb.com/NASApp/mlb/index.jsp?c_id=mil",
		"http://minnesota.twins.mlb.com/NASApp/mlb/index.jsp?c_id=min",
		"http://montreal.expos.mlb.com/mon/home.html",
		"http://newyork.mets.mlb.com/NASApp/mlb/index.jsp?c_id=nym",
		"http://newyork.yankees.mlb.com/NASApp/mlb/index.jsp?c_id=nyy",
		"http://oakland.athletics.mlb.com/NASApp/mlb/index.jsp?c_id=oak",
		"http://philadelphia.phillies.mlb.com/NASApp/mlb/index.jsp?c_id=phi",
		"http://pittsburgh.pirates.mlb.com/NASApp/mlb/index.jsp?c_id=pit",
		"http://sandiego.padres.mlb.com/NASApp/mlb/index.jsp?c_id=sd",
		"http://sanfrancisco.giants.mlb.com/NASApp/mlb/index.jsp?c_id=sf",
		"http://seattle.mariners.mlb.com/NASApp/mlb/index.jsp?c_id=sea",
		"http://stlouis.cardinals.mlb.com/NASApp/mlb/index.jsp?c_id=stl",
		"http://tampabay.devilrays.mlb.com/NASApp/mlb/index.jsp?c_id=tb",
		"http://texas.rangers.mlb.com/NASApp/mlb/index.jsp?c_id=tex",
		"http://toronto.bluejays.mlb.com/NASApp/mlb/index.jsp?c_id=tor"
	};

	String[] teamNames = {
		"ANAHEIM",
		"ARIZONA",
		"ATLANTA",
		"BALTIMORE",
		"BOSTON",
		"CHICAGO CUBS",
		"CHI WHITE SOX",
		"CINCINNATI",
		"CLEVELAND",
		"COLORADO",
		"DETROIT",
		"FLORIDA",
		"HOUSTON",
		"KANSAS CITY",
		"LOS ANGELES",
		"MILWAUKEE",
		"MINNESOTA",
		"MONTREAL",
		"NY METS",
		"NY YANKEES",
		"OAKLAND",
		"PHILADELPHIA",
		"PITTSBURGH",
		"SAN DIEGO",
		"SAN FRANCISCO",
		"SEATTLE",
		"ST LOUIS",
		"TAMPA BAY",
		"TEXAS",
		"TORONTO",
	};
	
	Hashtable links;
	/**
	 * This method does the work of retrieving the data from the BLS web site and converting it into
	 * KavaChart Dataset classes.  It's pretty simple, really.  Just a bunch of stuff to generate
	 * and parse the page.
	 */
	private void generateNL(){
		if(goLive){
		Vector nationalLeagueEastStrings = new Vector();
		Vector nationalLeagueCentralStrings = new Vector();
		Vector nationalLeagueWestStrings = new Vector();

		try{
		String nbaURLString = "http://www.usatoday.com/sports/baseball/nl/standings.htm";
		URL nbaURL = new URL(nbaURLString);
		BufferedReader reader = new BufferedReader(new InputStreamReader(nbaURL.openStream()));
		String line = reader.readLine(); 
		while(!line.startsWith(startTag)){ //strip off header
			line = reader.readLine();
		}
		for(int i=0;i<5;i++)
			reader.readLine();
		//atlantic div
		for(int i=0;i<5;i++)
			nationalLeagueEastStrings.addElement(reader.readLine());
		//header stuff
		for(int i=0;i<8;i++)
			reader.readLine();
		//central div
		for(int i=0;i<6;i++)
			nationalLeagueCentralStrings.addElement(reader.readLine());
		//header stuff
		for(int i=0;i<8;i++)
			reader.readLine();
		for(int i=0;i<5;i++)
			nationalLeagueWestStrings.addElement(reader.readLine());
		}
		catch(IOException io){
			return;
		}
		datasets.addElement(buildDataset("NL East", nationalLeagueEastStrings));
		datasets.addElement(buildDataset("NL Central", nationalLeagueCentralStrings));
		datasets.addElement(buildDataset("NL West", nationalLeagueWestStrings));
		}
		else{
			datasets.addElement(buildDataset("NL East", NL_EastStrings));
			datasets.addElement(buildDataset("NL Central", NL_CentralStrings));
			datasets.addElement(buildDataset("NL West", NL_WestStrings));
		}

	}
	
	private void generateAL(){
		if(goLive){
		Vector americanLeagueEastStrings = new Vector();
		Vector americanLeagueCentralStrings = new Vector();
		Vector americanLeagueWestStrings = new Vector();

		try{
		String nbaURLString = "http://www.usatoday.com/sports/baseball/al/standings.htm";
		URL nbaURL = new URL(nbaURLString);
		BufferedReader reader = new BufferedReader(new InputStreamReader(nbaURL.openStream()));
		String line = reader.readLine(); 
		while(!line.startsWith(startTag)){ //strip off header
			line = reader.readLine();
		}
		for(int i=0;i<5;i++)
			reader.readLine();
		//atlantic div
		for(int i=0;i<5;i++)
			americanLeagueEastStrings.addElement(reader.readLine());
		//header stuff
		for(int i=0;i<8;i++)
			reader.readLine();
		//central div
		for(int i=0;i<5;i++)
			americanLeagueCentralStrings.addElement(reader.readLine());
		//header stuff
		for(int i=0;i<8;i++)
			reader.readLine();
		for(int i=0;i<4;i++)
			americanLeagueWestStrings.addElement(reader.readLine());
		}
		catch(IOException io){
			return;
		}
		datasets.addElement(buildDataset("AL East", americanLeagueEastStrings));
		datasets.addElement(buildDataset("AL Central", americanLeagueCentralStrings));
		datasets.addElement(buildDataset("AL West", americanLeagueWestStrings));
		}else{
			datasets.addElement(buildDataset("AL East", AL_EastStrings));
			datasets.addElement(buildDataset("AL Central", AL_CentralStrings));
			datasets.addElement(buildDataset("AL West", AL_WestStrings));
		}

	}
	
	private void generate(){
		datasets = new Vector();
		if(doNL)
			generateNL();
		if(doAL)
			generateAL();
		buildLinks();
	}
	
	public void setDoAL(boolean yesNo){
		doAL = yesNo;
	}
	
	public void doNL(boolean yesNo){
		doNL = yesNo;
	}
	
	public String[] getTeamsInDivision(String divisionName){
		if(datasets==null)
			generate();
		Enumeration en = datasets.elements();
		while(en.hasMoreElements()){
			Dataset d = (Dataset)en.nextElement();
			if(d.getName().equals(divisionName)){
				ArrayList list = new ArrayList();
				for(int i=0;i<d.getData().size();i++){
					list.add(d.getDataElementAt(i).getLabel());
				}
				Object[] objs = list.toArray();
				String[] s = new String[objs.length];
				for(int j=0;j<s.length;j++){
					s[j] = (String)objs[j];
				}
				return s;
			}
		}
		return null;
	}
	
	public DataProvider getOneDivisionDataProvider(String teamName){
		return new OneDivisionDataProvider(teamName, this);
	}
	
	class OneDivisionDataProvider implements DataProvider {
		String divisionName = null;
		NLALSectormapData data;
		
		OneDivisionDataProvider(String divisionName, NLALSectormapData data){
			this.divisionName = divisionName;
			this.data = data;
		}
		
		public Enumeration getDatasets(){
			Enumeration sets = data.getDatasets();
			System.out.println("looking for " + divisionName);
			while(sets.hasMoreElements()){
				Dataset d = (Dataset)sets.nextElement();
				System.out.println("name is " + d.getName() +  ", " + d.getDataElementAt(0).getLabel());
				if(d.getName().equalsIgnoreCase(divisionName)){
					ArrayList list = new ArrayList();
					list.add(d);
					return Collections.enumeration(list);
				}
			}
			System.out.println("returning null");
			return null;
		}
		
		public String getUniqueIdentifier(){
			return divisionName + data.getUniqueIdentifier();
		}
	}
	
	private void buildLinks(){
		links = new Hashtable();
		for(int i=0;i<teamLinks.length;i++){
			links.put(teamNames[i], teamLinks[i]);
		}
	}
	
	public String getDatasetLinks(int setNum){
		if(datasets==null)
			getDatasets();
		StringBuffer sb = new StringBuffer();
		Dataset d = (Dataset)datasets.elementAt(setNum);
		Enumeration en = d.getData().elements();
		while(en.hasMoreElements()){
			Datum dat = (Datum)en.nextElement();
			sb.append(links.get(dat.getLabel()));
			sb.append(",");
		}
		return sb.toString();
	}
	
	public String getDatasetLinks(String divisionName){
		if(datasets==null)
			getDatasets();
		StringBuffer sb = new StringBuffer();
		Dataset dataset = null;
		Enumeration en = datasets.elements();
		while(en.hasMoreElements()){
			Dataset d = (Dataset)en.nextElement();
			if(d.getName().equals(divisionName))
				dataset = d;
		}
		if(dataset==null)
			return null;
		en = dataset.getData().elements();
		while(en.hasMoreElements()){
			Datum dat = (Datum)en.nextElement();
			sb.append(links.get(dat.getLabel()));
			sb.append(",");
		}
		return sb.toString();
	}
	
	private Dataset buildDataset(String division, String[] strings){
		Vector v = new Vector();
		for(int i=0;i<strings.length;i++){
			v.addElement(strings[i]);
		}
		return buildDataset(division, v);
	}
	
	final static String WON = "WON";
	final static String LOST = "LOST";

	private Dataset buildDataset(String division, Vector teams){
		Dataset d = new Dataset();
		d.setName(division);
		Enumeration en = teams.elements();
		//System.out.println("static String[] " + division + "Strings = {");
		while(en.hasMoreElements()){
			String team = (String)en.nextElement();
			//System.out.println("\t\"" + team + "\",");
			if(team.length()>1){
				String name = team.substring(0, 14).trim();
				if(name.startsWith("X-"))
					name=name.substring(2);
				else if(name.startsWith("Y-"))
					name = name.substring(2);
				team = team.substring(15);
				StringTokenizer st = new StringTokenizer(team);
				String wins = st.nextToken();
				String losses = st.nextToken();
				String percentage = st.nextToken();
				double streak = 0.0;
				int streakLoc = team.indexOf(WON);
				if(streakLoc!=-1){
					String s = team.substring(streakLoc + WON.length()).trim();
					streak = Double.parseDouble(s);
				}
				else{
					streakLoc = team.indexOf(LOST);
					if(streakLoc!=-1){
						String s = team.substring(streakLoc + LOST.length()).trim();
						streak = 0.0 - Double.parseDouble(s);
					}					
				}
				//d.addDatum(new Datum(streak, Double.parseDouble(percentage), name, 0, null));
				//System.out.println("wins:" + wins + " losses:" + losses + " streak:" + streak);
				//d.addDatum(new Datum(streak, Double.parseDouble(wins), Double.parseDouble(losses), name, 0, null));
				d.addDatum(new Datum(streak, Double.parseDouble(percentage), Double.parseDouble(wins), name, 0, null));
			}
		}
		//System.out.println("};");
		return d;
	}

	final public static long SIX_HOURS = 1000 * 60 * 60 * 6;
	public Enumeration getDatasets(){
		Date d = new Date();
		if((lastRetrieved==null)||(d.getTime()-lastRetrieved.getTime()>SIX_HOURS)){
			lastRetrieved = new Date();
			generate();
		}
		return datasets.elements();
	}
	public String getUniqueIdentifier(){
		Date d = new Date();
		if((lastRetrieved==null)||(d.getTime()-lastRetrieved.getTime()>SIX_HOURS)){
			lastRetrieved = new Date();
			generate();
		}
		return lastRetrieved.toString();
	}

	//for unit testing - make sure the site is alive and retrieves values for us
	public static void main(String[] args){
		NLALSectormapData data = new NLALSectormapData();

		DataProvider dp = data.getOneDivisionDataProvider("AL East");	
		Enumeration e = dp.getDatasets();
		
		while(e.hasMoreElements()){
			Dataset d = (Dataset)e.nextElement();
			System.out.println("got " + d.getName());
			System.out.println("with " + d.getData().size() + " elements");
			for(int i=0;i<d.getData().size();i++){
				System.out.println("dat:" + d.getDataElementAt(i).getLabel() + ", " + d.getDataElementAt(i).getX() + ", " + d.getDataElementAt(i).getY());
			}
		}
		
		System.out.println("------------");
		dp = data.getOneDivisionDataProvider("AL East");	
		e = dp.getDatasets();
		while(e.hasMoreElements()){
			Dataset d = (Dataset)e.nextElement();
			System.out.println("got " + d.getName());
			System.out.println("with " + d.getData().size() + " elements");
			for(int i=0;i<d.getData().size();i++){
				System.out.println("dat:" + d.getDataElementAt(i).getLabel() + ", " + d.getDataElementAt(i).getX() + ", " + d.getDataElementAt(i).getY());
			}
		}
	}
	
	public String getGamesBack(String teamName){
		if(datasets==null)
			generate();
		Enumeration en = datasets.elements();
		while(en.hasMoreElements()){
			Dataset d = (Dataset)en.nextElement();
			int leaderWins = (int)d.getDataElementAt(0).getY();
			Enumeration data = d.getData().elements();
			while(data.hasMoreElements()){
				Datum datum = (Datum)data.nextElement();
				if(datum.getLabel().equals(teamName)){
					int gamesBack = leaderWins - (int)datum.getY();
					if(gamesBack==0)
						return " ";
					return Integer.toString(gamesBack);
				}
			}
		}
		return " "; // not found??
	}
}
