/*
* Copyright 1997-2004 Visual Engineering, Inc. All Rights Reserved.
*
* This software is the proprietary information of Visual Engineering, Inc.
* Use is subject to license terms.
*/
package examples;

import java.net.*;
import java.util.*;
import java.io.*;

import com.ve.kavachart.chart.*;
import com.ve.kavachart.utility.*;

public class NewYorkWeather implements DataProvider {

	Vector datasets = new Vector();
	boolean noisy = false;
	Date lastChecked = null;

	public void generate(){
		System.out.println("getting new york weather info");
		lastChecked = new Date();
		datasets.removeAllElements();
		Vector cityNames = new Vector();
		Vector temperatures = new Vector();
		try{
		URL noaaURL = new URL("http://www.srh.noaa.gov/data/NYC/SWRNY");
		BufferedReader reader = new BufferedReader(new InputStreamReader(noaaURL.openStream()));
		String line = reader.readLine(); 
		while(!line.startsWith("NEW YORK CITY")){ //ignore top level summary
			line = reader.readLine();
		}
		reader.readLine(); //strip off one more line
		reader.readLine(); //strip off one more line

		line = reader.readLine();
		while((line!=null)&&(!line.startsWith("$$"))){ //read interesting stuff
			String cityName = line.substring(0, 15).trim();
			String temperature = line.substring(25, 27).trim();
			if((!temperature.equals("MM"))&&(temperature.indexOf("A")==-1)&&(temperature.length()>0)){ //temp is available...
				temperatures.addElement(temperature);
				cityNames.addElement(cityName);
			}
			line = reader.readLine();
		}
		}
		catch(IOException io){
			return;
		}

		Dataset dataset = new Dataset();
		dataset.setName("Bay Area Temperatures:" + lastChecked);
		for(int i=0;i<temperatures.size();i++){
			String temp = (String)temperatures.elementAt(i);
			String city = (String)cityNames.elementAt(i);
			Datum dat = new Datum(Double.valueOf(temp).doubleValue(), city, i, null);
			dataset.addDatum(dat);
		}
		datasets.addElement(dataset);
	}

	static private long HALF_HOUR = 1000 * 60 * 30;
	public Enumeration getDatasets(){
		Date now = new Date();
		if((datasets.size()==0)||(now.getTime()-lastChecked.getTime()>HALF_HOUR))
			generate();
		return datasets.elements();
	}

	public String getUniqueIdentifier(){
		if(lastChecked==null)
			generate();
		return Long.toString(lastChecked.getTime());
	}

	//for unit testing - make sure the site is alive and retrieves values for us
	public static void main(String[] args){
		NewYorkWeather data = new NewYorkWeather();
		Enumeration e = data.getDatasets();
		Dataset d = (Dataset)e.nextElement();
		for(int i=0;i<d.getData().size();i++){
			System.out.println(i + ":" + d.getDataElementAt(i).getLabel() + " - " + d.getDataElementAt(i).getY());
		}
	}
}
