/*
* Copyright 1997-2004 Visual Engineering, Inc. All Rights Reserved.
*
* This software is the proprietary information of Visual Engineering, Inc.
* Use is subject to license terms.
*/
package examples;

import java.net.*;
import java.util.*;
import java.io.*;
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;

import com.ve.kavachart.utility.*;
import com.ve.kavachart.chart.*;

public class UI_MontereyBouyData extends JPanel implements DataProvider, ActionListener {

	Vector datasets;
	boolean noisy = false;
	Date lastChecked = null;

	public UI_MontereyBouyData(){
		initUI();
	}

	Dataset windGust, windSpeed, waveHeight, waterTemp, airTemp;
	Double discontinuity = new Double(Double.NEGATIVE_INFINITY);
	
	public void generate(){

		System.out.println("reading data from monterey bouy");

		datasets.removeAllElements();
		Vector fileLines = new Vector();
		lastChecked = new Date();

		try{
			URL noaaURL = new URL("http://www.ndbc.noaa.gov/data/realtime2/46042.txt");
			BufferedReader reader = new BufferedReader(new InputStreamReader(noaaURL.openStream()));
			String line = reader.readLine(); //strip off the titles...
			line = reader.readLine();
			while(line!=null){
				fileLines.addElement(line);
				line = reader.readLine();
			}
			reader.close();
		}
		catch(Exception e){
		}

		Vector vWindGust = new Vector();
		Vector vWindSpeed = new Vector();
		Vector vTimeStamps = new Vector();
		Vector vWaveHeight = new Vector();
		Vector vWaterTemp = new Vector();
		Vector vAirTemp = new Vector();

		int hoursOfData = fileLines.size()/4;
		java.text.SimpleDateFormat parser = (java.text.SimpleDateFormat)java.text.DateFormat.getInstance();
		parser.applyPattern("yyyyMMddHH");

		for(int i=hoursOfData-1;i>-1;i--){
			String thisLine = (String)fileLines.elementAt(i);
			StringTokenizer st = new StringTokenizer(thisLine);
			String ts = st.nextToken() + st.nextToken() + st.nextToken() + st.nextToken();
			try{
			vTimeStamps.addElement(parser.parse(ts));
			}catch(Exception bah){
				System.out.println("bad timestamp");
			}
			st.nextToken();

			try{
				Double windSpeed = new Double(((Double.valueOf(st.nextToken()).doubleValue() * 60 * 60) / 1000) * 0.6214);
				vWindSpeed.addElement(windSpeed);
			}catch(Exception e){
				vWindSpeed.addElement(discontinuity);
			}

			try{
				Double windGust = new Double(((Double.valueOf(st.nextToken()).doubleValue() * 60 * 60) / 1000) * 0.6214);
				vWindGust.addElement(windGust);
			}catch(Exception e){
				vWindGust.addElement(discontinuity);
			}

			try{
				Double waveHt = new Double(((Double.valueOf(st.nextToken()).doubleValue() * 100)/2.54)/12);
				vWaveHeight.addElement(waveHt);
			}catch(Exception e){
				vWaveHeight.addElement(discontinuity);
			}

			//skip some stuff...
			st.nextToken();st.nextToken();st.nextToken();st.nextToken();
			try{
				Double airTemp = new Double((Double.valueOf(st.nextToken()).doubleValue() * 9/5 ) + 32.);
				vAirTemp.addElement(airTemp);
			}catch(Exception e){
				vAirTemp.addElement(discontinuity);
			}

			try{
				Double waterTemp = new Double((Double.valueOf(st.nextToken()).doubleValue() * 9/5 ) + 32.);
				vWaterTemp.addElement(waterTemp);
			}catch(Exception e){
				vWaterTemp.addElement(discontinuity);
			}

		}

		//now turn all this data into KavaChart Dataset classes and add the classes to our Vector "datasets"
		windGust = new Dataset();
		windGust.setName("Wind Gust");
		windGust.getGc().setFillColor(java.awt.Color.blue);
		windGust.getGc().setLineColor(java.awt.Color.blue);
		populateDataset(windGust, vTimeStamps, vWindGust);

		windSpeed = new Dataset();
		windSpeed.setName("Wind Speed");
		windSpeed.getGc().setFillColor(java.awt.Color.green);
		windSpeed.getGc().setLineColor(java.awt.Color.green);
		populateDataset(windSpeed, vTimeStamps, vWindSpeed);

		waveHeight = new Dataset();
		waveHeight.setName("Wave Height");
		waveHeight.getGc().setFillColor(java.awt.Color.orange);
		waveHeight.getGc().setLineColor(java.awt.Color.orange);
		populateDataset(waveHeight, vTimeStamps, vWaveHeight);

		waterTemp = new Dataset();
		waterTemp.setName("Water Temp");
		waterTemp.getGc().setFillColor(java.awt.Color.green.darker());
		waterTemp.getGc().setLineColor(java.awt.Color.green.darker());
		populateDataset(waterTemp, vTimeStamps, vWaterTemp);
		datasets.addElement(waterTemp);

		airTemp = new Dataset();
		airTemp.getGc().setFillColor(java.awt.Color.gray);
		airTemp.getGc().setLineColor(java.awt.Color.gray);
		airTemp.setName("Air Temp");
		populateDataset(airTemp, vTimeStamps, vAirTemp);

	}

	private void populateDataset(Dataset d, Vector times, Vector vals){
		for(int i=0;i<times.size();i++){
			Date date = (Date)times.elementAt(i);
			Double val = (Double)vals.elementAt(i);
			if(val!=discontinuity)
				d.addPoint((double)date.getTime(), val.doubleValue(), null);
		}
	}

	static private long HOUR = 1000 * 60 * 60;
	public Enumeration getDatasets(){
		Date now = new Date();
		if((datasets==null)||(now.getTime()-lastChecked.getTime()>HOUR)){
			datasets = new Vector();
			generate();
		}
		
		datasets.removeAllElements();
		
		if(getParent()==null||windDataCheckBox.isSelected())
			datasets.addElement(windSpeed);
		if(getParent()==null||gustDataCheckBox.isSelected())
			datasets.addElement(windGust);
		if(getParent()==null||waterTempCheckBox.isSelected())
			datasets.addElement(waterTemp);
		if(getParent()==null||airTempCheckBox.isSelected())
			datasets.addElement(airTemp);
		if(getParent()==null||waveHeightCheckBox.isSelected())
			datasets.addElement(waveHeight);
			
		if(datasets.size()==0){
			windDataCheckBox.setSelected(true);
			datasets.addElement(windSpeed);
		}
		System.out.println("returning " + datasets.size() + " datasets");
			
		return datasets.elements();
	}

	public String getUniqueIdentifier(){
		if(lastChecked==null)
			generate();
		return Long.toString(lastChecked.getTime());
	}

	//for unit testing - make sure the site is alive and retrieves values for us
	public static void main(String[] args){
		UI_MontereyBouyData data = new UI_MontereyBouyData();
		Enumeration e = data.getDatasets();
		Dataset d = (Dataset)e.nextElement();
		for(int i=0;i<d.getData().size();i++){
			System.out.println(i + ":" + new Date((long)d.getDataElementAt(i).getX()) + " - " + d.getDataElementAt(i).getY());
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent arg0) {
		firePropertyChange(null, null, null);
	}
	
	JCheckBox windDataCheckBox, gustDataCheckBox, waveHeightCheckBox, airTempCheckBox, waterTempCheckBox;
	
	private void initUI(){
		windDataCheckBox = new JCheckBox("Wind Speed");
		gustDataCheckBox = new JCheckBox("Wind Gust Speed");
		waveHeightCheckBox = new JCheckBox("Wave Heights");
		airTempCheckBox = new JCheckBox("Air Temperatures");
		waterTempCheckBox = new JCheckBox("Water Temperatures");
		
		GridBagLayout gbl = new GridBagLayout();
		setLayout(gbl);
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.WEST;
		
		gbc.insets.top = 20;
		gbc.gridy = 0;
		add(windDataCheckBox);
		gbl.setConstraints(windDataCheckBox, gbc);
		
		gbc.gridy++;
		add(gustDataCheckBox);
		gbl.setConstraints(gustDataCheckBox, gbc);
		
		gbc.gridy++;
		add(waveHeightCheckBox);
		gbl.setConstraints(waveHeightCheckBox, gbc);
		
		gbc.gridy++;
		add(airTempCheckBox);
		gbl.setConstraints(airTempCheckBox, gbc);
		
		gbc.gridy++;
		add(waterTempCheckBox);
		gbl.setConstraints(waterTempCheckBox, gbc);
		
		windDataCheckBox.setSelected(true);
		
		windDataCheckBox.addActionListener(this);
		gustDataCheckBox.addActionListener(this);
		waveHeightCheckBox.addActionListener(this);
		airTempCheckBox.addActionListener(this);
		waterTempCheckBox.addActionListener(this);
		
	}
}