/*
 * Created on Jun 16, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
 
package examples;

import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

import java.io.*;
import java.net.*;
import java.util.*;
 
import com.ve.kavachart.chart.*;
import java.awt.*;
import javax.swing.*;

/**
 * @author David McMurdie
 *
 * Takes input data in XML format to produce KavaChart datasets.
 */
public class UI_XMLDataProvider extends JPanel implements com.ve.kavachart.utility.DataProvider {
	ArrayList datasets = null;
	Dataset currentDataset = null;
	double scaleFactor = 1.;
	String serviceURL = "http://www.kavachart.com/weather.jsp";
	private int currentDat = 0;
	java.text.SimpleDateFormat dateFormat = null;
	
	String instructions = "Specify a URL that provides data in the format specified by the KavaChartDataProvider DTD, " +
		"located at http://www.ve.com/kavachart/KavaChartDataProvider.dtd. " +
		"\n" +
		"A couple of examples:\n" +
		"http://www.kavachart.com/dat.xml\n" +
		"http://www.kavachart.com/weather.jsp\n";
	
	public UI_XMLDataProvider(){
		initUI();
	}
	
	public UI_XMLDataProvider(String serviceURL){
		this.serviceURL = serviceURL;
	}
	
	JTextField textField = new JTextField(25);
	JLabel label = new JLabel("Service URL");
	void initUI(){
		GridBagLayout gbl = new GridBagLayout();
		setLayout(gbl);
		add(label);
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.gridwidth = 1;
		gbl.setConstraints(label, constraints);

		add(textField);
		constraints.gridx = 1;
		gbl.setConstraints(textField, constraints);
		
		textField.setText(serviceURL);
		JTextArea ta = new JTextArea(5, 30);
		ta.setLineWrap(true);
		ta.setWrapStyleWord(true);
		ta.setText(instructions);
		ta.setEditable(false);
		add(ta);
		constraints.insets.top = 15;
		constraints.gridx = 0;
		constraints.gridy = 1;
		constraints.gridwidth = 2;
		gbl.setConstraints(ta, constraints);
	}
	
	public void setServiceURL(String serviceURL){
		this.serviceURL = serviceURL;
	}
	
	public String getServiceURL(){
		return serviceURL;
	}
	
	public void generate(){
		datasets =  new ArrayList();
		try{
		SAXParserFactory factory = SAXParserFactory.newInstance();
		SAXParser parser = factory.newSAXParser();
		
		parser.parse(getInputStream(), new MyHandler());
		}catch(Exception anything){
			anything.printStackTrace();
		}
	}
	
	private InputStream getInputStream(){
		try{
			URL url = new URL(serviceURL);
			return url.openConnection().getInputStream();
		}catch(Exception anything){
			System.out.println("problem with this URL:" + serviceURL);
		}
		return null;
	}
	
	public static void main(String[] args){
		UI_XMLDataProvider test = new UI_XMLDataProvider();
		Enumeration en = test.getDatasets();
		while(en.hasMoreElements()){
			Dataset d = (Dataset)en.nextElement();
			System.out.println("name:" + d.getName());
			Enumeration points = d.getData().elements();
			while(points.hasMoreElements()){
				Datum point = (Datum)points.nextElement();
				System.out.println("point:" + point.getX() + ", " + point.getY());
			}
		}		
	}
	
	class MyHandler extends DefaultHandler {
		UI_XMLDataProvider parent = null;
		boolean getDatasetLabel = false;
		
		public void startElement(String uri,
								String localName,
								String qName,
								Attributes attributes)
								throws SAXException{
			handleStartElement(qName, attributes);
		}
		
		public void endElement(String uri,
								String localName,
								String qName)
								throws SAXException{
			handleEndElement(qName);
		}
		
		private void handleStartElement(String name, Attributes attributes)throws SAXException {
			if(name.equals("dataset")){
				currentDataset = new Dataset();
				currentDataset.setName(attributes.getValue("name"));
				String color = attributes.getValue("color");
				if(color!=null){
					currentDataset.getGc().setFillColor(getColor(color));
					currentDataset.getGc().setLineColor(getColor(color));
				}
				currentDat = 0;
			}
			else if(name.equals("datum")){
				try{
					double y = parseDouble(attributes.getValue("y"));
					Datum dat = new Datum(currentDat++, y, null);
					String s = attributes.getValue("x");
					if(s!=null)
						dat.setX(parseDouble(s));
					s = attributes.getValue("y2");
					if(s!=null)
						dat.setY2(parseDouble(s));
					s = attributes.getValue("y3");
					if(s!=null)
						dat.setY3(parseDouble(s));
					s = attributes.getValue("date");
					if(s!=null){
						if(dateFormat!=null)
							dat.setX(dateFormat.parse(s).getTime());
						else
							dat.setX(Date.parse(s));
					}
						
					s = attributes.getValue("label");
					if(s!=null)
						dat.setLabel(s);
					s = attributes.getValue("color");
					if(s!=null){
						dat.getGc().setFillColor(getColor(s));
						dat.getGc().setLineColor(getColor(s));
					}
					
					currentDataset.addDatum(dat);
				}catch(Exception e){
					throw new SAXException("problem parsing Datum attribute");
				}
			}
			else if(name.equals("root")){
				String s = attributes.getValue("dateFormat");
				if(s!=null)
					dateFormat = new java.text.SimpleDateFormat(s);
			}
		}

		private void handleEndElement(String name){
			if(name.equals("dataset")){
				datasets.add(currentDataset);
				currentDataset = null;
			}
		}
		
		private double parseDouble(String s){
			try{
				return Double.parseDouble(s);
			}catch(Exception e){
				return Datum.DEFAULT;
			}
		}
	}
	/* (non-Javadoc)
	 * @see com.ve.kavachart.utility.DataProvider#getDatasets()
	 */
	public Enumeration getDatasets() {
		if(!serviceURL.equals(textField.getText())){
			datasets = null;
			serviceURL = textField.getText();
		}
		try{
		if(datasets==null)
			generate();
		}catch(Exception e){
			JOptionPane.showMessageDialog(new JFrame(), "Problem with service URL, using default data", "Web Service Unavailable",
				JOptionPane.WARNING_MESSAGE);
			serviceURL = "http://www.kavachart.com/dat.xml";
			textField.setText(serviceURL);
			generate();
		}
		return Collections.enumeration(datasets);
	}

	/* (non-Javadoc)
	 * @see com.ve.kavachart.utility.DataProvider#getUniqueIdentifier()
	 */
	public String getUniqueIdentifier() {
		return serviceURL;
	}
	
	//utility method from ParameterParser
	public Color getColor(String s) {
		if (s.equalsIgnoreCase("black"))
			return Color.black;
		if (s.equalsIgnoreCase("white"))
			return Color.white;
		if (s.equalsIgnoreCase("lightGray"))
			return Color.lightGray;
		if (s.equalsIgnoreCase("gray"))
			return Color.gray;
		if (s.equalsIgnoreCase("darkGray"))
			return Color.darkGray;
		if (s.equalsIgnoreCase("red"))
			return Color.red;
		if (s.equalsIgnoreCase("pink"))
			return Color.pink;
		if (s.equalsIgnoreCase("orange"))
			return Color.orange;
		if (s.equalsIgnoreCase("yellow"))
			return Color.yellow;
		if (s.equalsIgnoreCase("green"))
			return Color.green;
		if (s.equalsIgnoreCase("magenta"))
			return Color.magenta;
		if (s.equalsIgnoreCase("cyan"))
			return Color.cyan;
		if (s.equalsIgnoreCase("blue"))
			return Color.blue;
		if (s.equalsIgnoreCase("transparent"))
			return Gc.TRANSPARENT;
		try {
			if(s.startsWith("0x"))
				s = s.substring(2);
			return new Color(Integer.parseInt(s, 16));
		}
		catch (NumberFormatException e) {
		}
			return Color.black;
	}

}
