/*
* Copyright 1997-2003 Visual Engineering, Inc. All Rights Reserved.
*
* This software is the proprietary information of Visual Engineering, Inc.
* Use is subject to license terms.
*/
package examples;

import java.net.*;
import java.util.*;
import java.io.*;
import java.text.*;

import com.ve.kavachart.utility.*;
import com.ve.kavachart.chart.*;

import javax.swing.*;
import java.awt.*;

public class UI_YahooCandlestickData extends JPanel implements DataProvider {

	Vector datasets;
	boolean noisy = false;
	int startMonth = 1, startDay = 1, startYear = 2004;
	int endMonth = 3, endDay = 1, endYear = 2004;
	String period = "d";
	String symbol = "SUNW";
	String yahooString;
	boolean useVolumeData = false;
	boolean reQuery = true;
	
	public UI_YahooCandlestickData(){
		initUI();
	}
	
	public boolean isGUI(){
		return getParent()!=null;
	}
	
	public void setUseVolumeData(boolean foo){
		if(useVolumeData!=foo)
			reQuery = true;
		useVolumeData = foo;
	}

	public void setSymbol(String symbol){
		if(this.symbol!=symbol)
			reQuery = true;
		this.symbol = symbol;
		//System.out.println("symbol:" + symbol);
	}

	public void setPeriod(String period){
		if(this.period!=period)
			reQuery = true;
		this.period = period;
		//System.out.println("period:" + period);
	}

	public void setStartMonth(int month){
		if(startMonth!=month)
			reQuery = true;
		startMonth = month;
		//System.out.println("startMonth:" + month);
	}

	public void setStartDay(int day){
		if(startDay!=day)
			reQuery = true;
		startDay = day;
		//System.out.println("startDay:" + day);
	}

	public void setStartYear(int year){
		if(startYear!=year)
			reQuery = true;
		startYear = year;
		//System.out.println("startYear:" + year);
	}

	public void setEndMonth(int month){
		if(endMonth!=month)
			reQuery = true;
		endMonth = month;
	}

	public void setEndDay(int day){
		if(endDay!=day)
			reQuery = true;
		endDay = day;
	}

	public void setEndYear(int year){
		if(endYear!=year)
			reQuery = true;
		endYear = year;
	}


	private String getYahooString(){
		if(isGUI()){
		yahooString = "http://table.finance.yahoo.com/table.csv?" +
			"a=" + startMonthComboBox.getSelectedIndex() + "&" +
			"b=" + startDayComboBox.getSelectedItem() + "&" + 
			"c=" + startYearComboBox.getSelectedItem() + "&" +
			"d=" + endMonthComboBox.getSelectedIndex() + "&" +
			"e=" + endDayComboBox.getSelectedItem() + "&" +
			"f=" + endYearComboBox.getSelectedItem() + "&" +
			"g=" + periodComboBox.getSelectedItem() + "&" +
			"s=" + symbolComboBox.getSelectedItem() +"&ignore=.csv";
		}else{
			yahooString = "http://table.finance.yahoo.com/table.csv?" +
				"a=" + startMonth + "&" +
				"b=" + startDay + "&" + 
				"c=" + startYear + "&" +
				"d=" + endMonth + "&" +
				"e=" + endDay + "&" +
				"f=" + endYear + "&" +
				"g=" + period + "&" +
				"s=" + symbol +"&ignore=.csv";
		}
		return yahooString;
	}

	public void generate()throws Exception {
		datasets = new Vector();

		yahooString = getYahooString();
		System.out.println("fetching from " + yahooString);
		/*
		** get some data from yahoo:
		** open a URL, read the lines out of it, reverse the lines, and put them into the chart
		*/
		
		URL yahooURL = new URL(yahooString);
		BufferedReader reader = new BufferedReader(new InputStreamReader(yahooURL.openStream()));

		Vector fileLines = new Vector();
		//we don't need the first line
		String line = reader.readLine();
		line = reader.readLine();
		while(line!=null){
			if(!line.startsWith("<!"))
				fileLines.addElement(line);
			line = reader.readLine();
		}

		//we need to reverse the file, since Yahoo's most recent observations are first, and
		//we want them last
		Vector newFileLines = new Vector();
		for(int i=fileLines.size()-1;i>-1;i--){
			newFileLines.addElement(fileLines.elementAt(i));
		}

		//get a time parser:
		DateFormat parser = DateFormat.getInstance();
		((SimpleDateFormat)parser).applyPattern("d-MMM-yy");

		//now accumulate the data into a Dataset
		Dataset priceData = new Dataset();
		priceData.setName("Price Data for " + symbol);
		priceData.getGc().setLineColor(java.awt.Color.blue);
		priceData.getGc().setFillColor(java.awt.Color.blue);
		Dataset volumeData = new Dataset();
		volumeData.setName("volume");
		volumeData.getGc().setLineColor(java.awt.Color.GREEN);
		volumeData.getGc().setFillColor(java.awt.Color.GREEN);
		fileLines = newFileLines;
		for(int i=0;i<fileLines.size();i++){
			line = (String)fileLines.elementAt(i);
			StringTokenizer lineTokens = new StringTokenizer((String)fileLines.elementAt(i),",");
			double dateVal = (double)(parser.parse(lineTokens.nextToken()).getTime());
			//chart.accumulateProperty("dataset0dateValues", lineTokens.nextToken());
			//chart.accumulateProperty("dataset0openValues", lineTokens.nextToken());
			double openVal = Double.parseDouble(lineTokens.nextToken());
			double highVal = Double.parseDouble(lineTokens.nextToken());
			double lowVal = Double.parseDouble(lineTokens.nextToken());
			double closeVal = Double.parseDouble(lineTokens.nextToken());
			double volVal = Double.parseDouble(lineTokens.nextToken());
			//chart.accumulateProperty("dataset0highValues", lineTokens.nextToken());
			//chart.accumulateProperty("dataset0lowValues", lineTokens.nextToken());
			//chart.accumulateProperty("dataset0closeValues", lineTokens.nextToken());
			//skip volume, which would be nextToken()
			priceData.getData().addElement(new CandlestickDatum(dateVal, highVal, lowVal, openVal, closeVal, null));
			volumeData.addPoint(dateVal, volVal, null);
		}
		datasets.addElement(priceData);
		if(useVolumeData)
			datasets.addElement(volumeData);
	}

	public Enumeration getDatasets(){
		if(isGUI()||reQuery){
			try{
				System.out.println("generating..." + (getParent()==null));
				generate();
				reQuery = false;
			}catch(Exception e){
				JOptionPane.showMessageDialog(new JFrame(), "Problem loading Yahoo Data", "Data Not Available",
					JOptionPane.ERROR_MESSAGE);
			}
		}
		return datasets.elements();
	}

	public String getUniqueIdentifier(){
		return getYahooString();
	}
	
	//now for the DataProviderFace interface
	JPanel panel;
	JComboBox symbolComboBox;
	JComboBox startMonthComboBox, startDayComboBox, startYearComboBox;
	JComboBox endMonthComboBox, endDayComboBox, endYearComboBox;
	JComboBox periodComboBox;

	private void initUI(){

		GridBagLayout gbl = new GridBagLayout();
		setLayout(gbl);
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 45;
		
		JLabel attributionLabel = new JLabel("Stock data from Yahoo!");
		add(attributionLabel);
		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.gridwidth = 2;
		gbl.setConstraints(attributionLabel, constraints);
		
		constraints.insets = new Insets(10, 10, 10, 10);
		JLabel nameLabel = new JLabel("Stock Symbol");
		add(nameLabel);
		constraints.gridx = 0;
		constraints.gridy++;
		constraints.gridwidth = 1;
		gbl.setConstraints(nameLabel, constraints);
		
		
		String[] symbols = {
			"SUNW", "MSFT", "IBM", "FDX", "GM", "YHOO"
		};
		symbolComboBox = new JComboBox(symbols);
		add(symbolComboBox);
		constraints.gridx = 1;
		constraints.gridwidth = 2;
		gbl.setConstraints(symbolComboBox, constraints);
		
		constraints.insets.top = 25;
		JLabel label = new JLabel("Data Interval");
		constraints.gridy++;
		constraints.gridx = 0;
		constraints.gridwidth = 1;
		add(label);
		gbl.setConstraints(label, constraints);
		
		String[] periods = {
			"d", "w", "m"
		};
		periodComboBox = new JComboBox(periods);
		add(periodComboBox);
		constraints.gridx = 1;
		gbl.setConstraints(periodComboBox, constraints);
		
		String[] months = {
			"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
		};
		String[] days = new String[31];
		for(int i=0;i<days.length;i++){
			days[i] = Integer.toString(i+1);
		}
		
		Date d = new Date();
		Calendar c = Calendar.getInstance();
		c.setTime(d);
		int year = c.get(Calendar.YEAR);
		int month = c.get(Calendar.MONTH);
		int day = c.get(Calendar.DAY_OF_MONTH);
		
		c.add(Calendar.MONTH, -2);
		int beginY = c.get(Calendar.YEAR);
		int beginM = c.get(Calendar.MONTH);
		int beginD = c.get(Calendar.DAY_OF_MONTH);
		
		String[] years = new String[8];
		for(int i=0;i<years.length;i++){
			years[i] = Integer.toString(year - years.length + i + 1);
		}
		
		constraints.insets.top = 0;
		constraints.insets.left = 0;

		JLabel startLabel = new JLabel("Start Date");
		add(startLabel);
		constraints.gridx = 0;
		constraints.gridy++;
		constraints.gridwidth = 1;
		gbl.setConstraints(startLabel, constraints);
		
		constraints.gridy++;
		startMonthComboBox = new JComboBox(months);
		add(startMonthComboBox);
		gbl.setConstraints(startMonthComboBox, constraints);
		
		constraints.gridx = 1;
		startDayComboBox = new JComboBox(days);
		add(startDayComboBox);
		gbl.setConstraints(startDayComboBox, constraints);
		
		constraints.gridx = 2;
		startYearComboBox = new JComboBox(years);
		add(startYearComboBox);
		gbl.setConstraints(startYearComboBox, constraints);
		
		JLabel endLabel = new JLabel("End Date");
		add(endLabel);
		constraints.gridx = 0;
		constraints.gridy++;
		gbl.setConstraints(endLabel, constraints);
		constraints.insets.top = 0;
		constraints.insets.left = 0;
		
		constraints.gridy++;
		endMonthComboBox = new JComboBox(months);
		add(endMonthComboBox);
		gbl.setConstraints(endMonthComboBox, constraints);
		
		constraints.gridx = 1;
		endDayComboBox = new JComboBox(days);
		add(endDayComboBox);
		gbl.setConstraints(endDayComboBox, constraints);
		
		constraints.gridx = 2;
		endYearComboBox = new JComboBox(years);
		add(endYearComboBox);
		gbl.setConstraints(endYearComboBox, constraints);
		
		constraints.insets.top = 25;
		constraints.gridy++;
		constraints.gridx = 0;
		constraints.gridwidth = 3;
		
		startDayComboBox.setSelectedIndex(beginD - 1);
		endDayComboBox.setSelectedIndex(day - 1);
		startMonthComboBox.setSelectedIndex(beginM);
		endMonthComboBox.setSelectedIndex(month);
		startYearComboBox.setSelectedItem(Integer.toString(beginY));
		endYearComboBox.setSelectedItem(Integer.toString(year));
		
		setStartYear(beginY);
		setStartDay(beginD - 1);
		setStartMonth(beginM);
		setEndYear(year);
		setEndMonth(month);
		setEndDay(day - 1);
		
	}
	public static void main(String[] args){
		UI_YahooCandlestickData dat = new UI_YahooCandlestickData();
		dat.getDatasets();
		dat.getDatasets();
	}
}
