import java.awt.*; 
import com.ve.kavachart.chart.*; 

public class ThresholdLine extends Line { 

	Transform dataTransform; 
	Double thresholdValue = null; 

	public ThresholdLine(){ 
		super(); 
	} 

	public ThresholdLine(double value){ 
		super(); 
		thresholdValue = new Double(value); 
	} 

	protected void doExtraordinaryIndicators(Graphics g, int series){ 
		Point circleSize = new Point(20,20); 
		Dataset d = datasets[series]; 
		for(int i=0;i<d.getData().size();i++){ 
			Datum dat = d.getDataElementAt(i); 
			if(dat instanceof ExtraordinaryDatum){ 
				Point p = dataTransform.point(dat.getX(), dat.getY()); 
				d.getGc().drawArc(g, p, circleSize, 0, 360);
				g.setColor(d.getLabelColor());
				ExtraordinaryDatum ed = (ExtraordinaryDatum)dat; 
				d.getGc().drawString(g, p.x, p.y, ed.getDescription()); 
			} 
		} 
	}

	public void draw(Graphics g){ 
		super.draw(g); 
		if(thresholdValue==null) 
			return; //nothing more to do 
		dataTransform = buildTransform();
		g = g.create(); 
		double startX = xAxis.getAxisStart(); 
		double endX = xAxis.getAxisEnd(); 
		Point rightPoint = dataTransform.point(startX, 
                                                        thresholdValue.doubleValue()); 
		Point leftPoint = dataTransform.point(endX, thresholdValue.doubleValue()); 
		g.clipRect(rightPoint.x, 0, leftPoint.x, 
				   globals.getMaxY() - leftPoint.y); 
		int i=0; 
		while(datasets[i]!=null){ 
			doThresholdLine(g, i); i++; 
		} 
		i = 0; 
		while(datasets[i]!=null){ 
			doExtraordinaryIndicators(g,i); 
			i++; 
		}
	} 
 
	public void setThreshold(double val){ 
		if(val == Double.NaN) 
			thresholdValue = null; 
		else thresholdValue = new Double(val); 
	} 

	public double getThreshold(){ 
		if(thresholdValue==null){
			return Double.NaN; 
		} 
		return thresholdValue.doubleValue(); 
	} 

	protected Transform buildTransform(){ 
		double xStart = xAxis.getAxisStart(); 
		double xEnd = xAxis.getAxisEnd(); 
		double yStart = yAxis.getAxisStart(); 
		double yEnd = yAxis.getAxisEnd();
                
		int plotRight = (int)(plotarea.getUrX() * globals.getMaxX()); 
		int plotLeft = (int)(plotarea.getLlX() * globals.getMaxX()); 
		int plotTop = (int)(plotarea.getUrY() * globals.getMaxY()); 
		int plotBase = (int)(plotarea.getLlY() * globals.getMaxY()); 
		return new Transform(xStart, yStart, xEnd, yEnd, 
                                        new Point(plotLeft, plotBase), 
                                        new Point(plotRight, plotTop)); 
	} 

	protected void doThresholdLine(Graphics g, int series){ 
		double[] xarr = datasets[series].getXValues(); 
		double[] yarr = datasets[series].getYValues(); 
		Point[] points = dataTransform.pointList(xarr,yarr); 
		Gc gc = datasets[series].getGc(); 
		Color saveColor = gc.getLineColor(); 
		gc.setLineColor(gc.getFillColor()); 
		gc.drawPolyline(g, points); 
		gc.setLineColor(saveColor); 
	} 

}
