import java.awt.*; 
import java.awt.event.*; 
import com.ve.kavachart.chart.*; 

class Zoomer implements MouseListener, MouseMotionListener { 

	Panel panel; 
	ChartInterface chart; 
	Point mousePressed; 
	Point mouseReleased; 
	int lastX, lastY; 
	DataTransform dataTransform; 

	public Zoomer(Panel panel, ChartInterface chart) { 
		this.panel = panel; 
		this.chart = chart; 
		panel.addMouseMotionListener(this); 
		panel.addMouseListener(this); 
	} 

	/** 
	 * test method 
	 */ 
	public static void main(String args[]) { 
		Frame f = new Frame(); 
		f.setSize(100, 100); 
		Panel p = new Panel(); 
		f.add(p); 
		Zoomer z = new Zoomer(p, null); 
		f.setVisible(true); 
	} 

	public void mouseClicked(MouseEvent e) { } 

	public void mouseDragged(MouseEvent e) { 
		Point p = e.getPoint(); 
		Graphics g = panel.getGraphics(); 
		g.setXORMode(Color.white); 
		g.drawLine(mousePressed.x, mousePressed.y, mousePressed.x, lastY);
		g.drawLine(mousePressed.x, lastY, lastX, lastY); 
		g.drawLine(lastX, lastY, lastX, mousePressed.y); 
		g.drawLine(mousePressed.x, mousePressed.y, lastX, mousePressed.y); 
		g.drawLine(mousePressed.x, mousePressed.y, mousePressed.x, p.y); 
		g.drawLine(mousePressed.x, p.y, p.x, p.y); 
		g.drawLine(p.x, p.y, p.x, mousePressed.y); 
		g.drawLine(mousePressed.x, mousePressed.y, p.x, mousePressed.y); 

		lastX = p.x; 
		lastY = p.y; 
	} 

	public void mouseEntered(MouseEvent e) { } 

	public void mouseExited(MouseEvent e) { } 

	public void mouseMoved(MouseEvent e) { } 
	
	public void mousePressed(MouseEvent e) { 
		if(e.isMetaDown()){ 
			resetChart(); 
			return; 
		} 
		mousePressed = e.getPoint(); 
		lastX = mousePressed.x; 
		lastY = mousePressed.y; 
	} 

	public void mouseReleased(MouseEvent e) { 
		mouseReleased = e.getPoint(); 
		zoomChart(); 
		panel.repaint(); 
	}

	/* 
	 * resets the chart axes to autoscaling 
	 */
	protected void resetChart() {   
		if(chart==null) 
			return; 
		AxisInterface axis; 
		axis = chart.getXAxis(); 
		axis.setAxisStart(Axis.AUTO_SCALE); 
		axis.setAxisEnd(Axis.AUTO_SCALE); 
		axis = chart.getYAxis(); 
		axis.setAxisStart(Axis.AUTO_SCALE); 
		axis.setAxisEnd(Axis.AUTO_SCALE); 
		//make sure mouseReleased doesn't re-zoom 
		mousePressed = null; 
	} 

	/* 
	 * zooms the chart by setting the axes to the mouse locations
	 */ 
	protected void zoomChart() { 
		if(mousePressed==null) 
			//set by resetChart 
			return; 
		if(dataTransform==null) 
			dataTransform = new DataTransform((Chart)chart); 
 
		Datum mouseDownDatum = dataTransform.pointToDatum(mousePressed); 
		Datum mouseUpDatum = dataTransform.pointToDatum(mouseReleased);
                
                double axisStart;
		double axisEnd; 
		AxisInterface axis;
                
                axisStart = Math.min(mouseDownDatum.getX(), mouseUpDatum.getX());
		axisEnd = Math.max(mouseDownDatum.getX(), mouseUpDatum.getX()); 
		axis = chart.getXAxis(); 
		axis.setAxisStart(axisStart); 
		axis.setAxisEnd(axisEnd); 
		axisStart = Math.min(mouseDownDatum.getY(), mouseUpDatum.getY()); 
		axisEnd = Math.max(mouseDownDatum.getY(), mouseUpDatum.getY()); 
		axis = chart.getYAxis(); 
		axis.setAxisStart(axisStart); 
		axis.setAxisEnd(axisEnd); 
	} 

}
