<%@ taglib uri="/WEB-INF/jsp/kavachart-taglib.tld" prefix="chart" %>
<head>
<title>KavaChart Line Chart Samples</title>
	<link type="text/css" rel="stylesheet" href="../../../documentation/style.css">
</head>
<P>
Line charts are one of the most basic and versatile chart types.  
They can be used to display trends over time, to correlate one variable 
with another, or to find clusters of information.  KavaChart's line chart 
collection includes charts that handle discontinuities, 
charts that have time axes, user-labeled axes, or numeric axes.
<p>
<p>
The KavaChart ProServe collection includes these line chart types:
<ul>
<li>lineApp
<li>disLineApp
<li>regressApp (scatter plots)
<li>labelLineApp
</ul>
Use the source code to this page as a guide in creating your own line charts.
You can also use the KavaChart <a href="http://www.ve.com/editor/" target="_blank">
ChartWizard</a>
to design your own line charts.
<hr>
<table border=0><tr><td valign=top>
This chart describes
oil contract prices over time.  In production, this chart's data parameters
would likely be created by a database connection, but we're just putting the strings in directly.
This means that in production the chart, like the rest of the data in your
page will be completely dynamic.
<p>
Placing your mouse over a region in the chart will show what the actual prices
are on a particular contract day.  The format of the numbers is defined using 
dwell properties.
<p>
A reference line at "12.5" describes our contract purchase price so we can tell at
a glance whether this contract is making money or not.
</td><td valign=top>
<chart:streamed chartType="lineApp" >
	<chart:param name="width" value="420" />
	<chart:param name="height" value="250" />
	<chart:param name="xAxisOptions" value="gridOn" />
	<chart:param name="yAxisOptions" value="rotateTitle, gridOn, tickOn, minTickOn" />
	<chart:param name="yAxisLabelPrecision" value="2" />
	<chart:param name="dwellXString" value="Contract Day: XX" />
	<chart:param name="dwellYString" value="Price USD: $XX" />
	<chart:param name="dwellYLabelPrecision" value="2" />
	<chart:param name="dataset0Name" value="Ptrole brut Brent" />
	<chart:param name="dataset0yValues" value="17.1,17,18.7,18.39,17.39,15.9,16.1,16.8,16.1,16.89,18,17.89,18,20,21,19.1,18.3,19.6,20.6,22.6,24.2,22.6,23.9,23.47,20.83,19.21,17.47,19.14,17.55,18.39,18.71,18.45,19.85,19,17,15.09,14.06,13.08,13.41,14.39,12.06,12.01,11.88,13.36,12.56,10.92,9.8,11.06,10.19,12.47,15.3,15.14" />
	<chart:param name="dataset0xValues" value="0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51" />
	<chart:param name="dataset0MarkerStyle" value="1" />
	<chart:param name="yAxisTitle" value="Price (USD)" />
	<chart:param name="xAxisTitle" value="Contract Day" />
	<chart:param name="legendOn" value="yes" />
	<chart:param name="yAxisThresholdLine0Value" value="12.5" />
	<chart:param name="yAxisThresholdLine0LabelString" value="Purchase Price" />
	<chart:param name="yAxisThresholdLine0Color" value="AAAAAA" />
</chart:streamed>
</td></tr></table>
<hr>
<table border=0><tr><td valign=top><tr><td>
<chart:streamed chartType="regressApp" >
	<chart:param name="antialiasOn" value="true" />
	<chart:param name="width" value="420" />
	<chart:param name="height" value="250" />
	<chart:param name="xAxisOptions" value="gridOn" />
	<chart:param name="yAxisOptions" value="rotateTitle, gridOn, tickOn, minTickOn" />
	<chart:param name="yAxisLabelPrecision" value="2" />
	<chart:param name="dwellXString" value="Contract Day: XX" />
	<chart:param name="dwellYString" value="Price USD: $XX" />
	<chart:param name="dwellYLabelPrecision" value="2" />
	<chart:param name="dataset0Name" value="Ptrole brut Brent" />
	<chart:param name="dataset0yValues" value="17.1,17,18.7,18.39,17.39,15.9,16.1,16.8,16.1,16.89,18,17.89,18,20,21,19.1,18.3,19.6,20.6,22.6,24.2,22.6,23.9,23.47,20.83,19.21,17.47,19.14,17.55,18.39,18.71,18.45,19.85,19,17,15.09,14.06,13.08,13.41,14.39,12.06,12.01,11.88,13.36,12.56,10.92,9.8,11.06,10.19,12.47,15.3,15.14" />
	<chart:param name="dataset0xValues" value="0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51" />
	<chart:param name="dataset0MarkerStyle" value="1" />
	<chart:param name="yAxisTitle" value="Price (USD)" />
	<chart:param name="xAxisTitle" value="Contract Day" />
	<chart:param name="legendOn" value="yes" />
	<chart:param name="yAxisThresholdLine0Value" value="12.5" />
	<chart:param name="yAxisThresholdLine0LabelString" value="Purchase Price" />
	<chart:param name="yAxisThresholdLine0Color" value="AAAAAA" />
</chart:streamed>
</td><td valign=top>
<p>
Using exactly the same parameters, but with "regressApp" instead of "lineApp", we get a
scatter plot and regression analysis of our data.
<p>
Either of these applets can be used to create scatter plots by adding the param "plotLinesOff",
while leaving line markers visible.  You can also select different markers, or even
choose an image ("dataset0Image") to use as a marker on the plot line.  Markers and lines
both appear on the chart's legend.
<p>
LineApp and regressApp both use numeric X and Y axes, which work well with many or few
points.  Numeric axes can also be scaled logarithmically (yAxisOptions|xAxisOptions=logAxis)
for data that covers extremely large ranges.
<p>
Note that we also enabled antialiasing to make a very smooth regression line.  A line
like this would appear quite jagged without antialiasing.
</td></tr></table>
<hr>
<table border=0><tr><td valign=top><tr><td valign=top>
Line charts can also include disontinuities, or breaks in the lines.  Discontinuities
provide a visual cue that a trend is interrupted by some significant event.
<p>
This sample is essentially the same as our first example, except that we've added
some discontinuities in the data by using "x" instead of a number in the list of
values for "dataset0yValues".  We also changed some colors, and re-arranged the axes
and legend.
<p>
We also changed the Y axis scaling to logarithmic.  For scientific
and financial data, it's sometimes important to see whether the data increases in
an exponential fashion, which appears as a more or less straight line with logarithmic
scaling.
<p>
Line chart variants that understand discontinuity handling include "disLineApp",
"disLabelLineApp".
</td><td valign=top>
<chart:streamed chartType="disLineApp" >
	<chart:param name="width" value="420" />
	<chart:param name="height" value="250" />
	<chart:param name="xAxisOptions" value="gridOn, tickOn, minTickOn" />
	<chart:param name="yAxisOptions" value="logScaling, rotateTitle, gridOn, tickOn, minTickOn, rightAxis" />
	<chart:param name="yAxisLabelPrecision" value="2" />
	<chart:param name="dwellUseDatasetName" value="true" />
	<chart:param name="dwellXString" value="Contract Day: XX" />
	<chart:param name="dwellYString" value="Price USD: $XX" />
	<chart:param name="dwellYLabelPrecision" value="2" />
	<chart:param name="dataset0Name" value="Ptrole brut Brent" />
	<chart:param name="titleString" value="60 Day Contract Prices" />
	<chart:param name="dataset0yValues" value="17.1,17,18.7,18.39,17.39,15.9,x,16.8,16.1,16.89,18,x,18,20,21,19.1,18.3,19.6,20.6,22.6,24.2,22.6,23.9,23.47,x,19.21,17.47,19.14,17.55,18.39,18.71,18.45,19.85,19,17,15.09,14.06,13.08,13.41,14.39,12.06,12.01,x,13.36,12.56,10.92,9.8,11.06,10.19,12.47,15.3,15.14" />
	<chart:param name="dataset0xValues" value="0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51" />
	<chart:param name="dataset0MarkerStyle" value="1" />
	<chart:param name="yAxisTitle" value="Price (USD)" />
	<chart:param name="xAxisTitle" value="Contract Day" />
	<chart:param name="legendOn" value="true" />
	<chart:param name="yAxisThresholdLine0Value" value="12.5" />
	<chart:param name="yAxisThresholdLine0LabelString" value="Purchase Price" />
	<chart:param name="yAxisThresholdLine0Color" value="AAAAAA" />
	<chart:param name="yAxisThresholdLine0LabelColor" value="AAAACC" />
</chart:streamed>
</td></tr></table>

<hr>
<table border=0><tr><td valign=top><tr><td valign=top>
<chart:streamed chartType="labelLineApp" >
	<chart:param name="width" value="400" />
	<chart:param name="height" value="200" />
	<chart:param name="colorPalette" value="web_seattle" />
	<chart:param name="defaultFont" value="Arial,12,0" />
	<chart:param name="dataset0MarkerStyle" value="0" />
	<chart:param name="dataset1MarkerStyle" value="1" />

	<chart:param name="dataset0Name" value="Gate 1" />

	<chart:param name="dataset0MarkerStyle" value="0" />
	<chart:param name="dataset0MarkerSize" value="6" />
	<chart:param name="dataset1Name" value="Gate 2" />
	<chart:param name="dataset1MarkerStyle" value="1" />
	<chart:param name="dataset1MarkerSize" value="8" />

	<chart:param name="yAxisOptions" value="gridOn,percentLabels,rotateTitle,leftAxis" />
	<chart:param name="yAxisTitle" value="Capacity" />

	<chart:param name="xAxisOptions" value="gridOn" />
	<chart:param name="xAxisLabels" value="-1 hr,-30,-15,Start,+15,+30,+1 hr,+2 hrs" />
	<chart:param name="legendOn" value="true" />
	<chart:param name="legendllX" value="0.56" />
	<chart:param name="legendllY" value="0.86" />

	<chart:param name="iconGap" value="0.02" />
	<chart:param name="dwellUseYValue" value="true" />
	<chart:param name="dwellYString" value="Percentage of Capacity: XX" />
	<chart:param name="dwellUseXValue" value="false" />
	<chart:param name="dwellUseLabelString" value="false" />
	<!-- Start of raw data sample from chart editor-->
	<chart:param name="dataset0yValues" value="0.49337174789927296,0.07637440555713904,0.6537596205613498,0.9785885121027812,0.7697164784656395,0.9999177752697896,0.8308754185175582,0.4144927925346378" />
	<chart:param name="dataset1yValues" value="0.06385574525260973,0.6617724669342112,0.4223733063604307,0.19948157131524946,0.10940961397098459,0.6371334633828918,0.452330509247856,0.823760202067575" />
</chart:streamed>
</td><td valign=top>
Another line chart variant is "labelLineApp".  This chart takes textual label definitions for the X axis.
If you don't define "xAxisLabels", they're derived from the first dataset's labels.  If those don't exist,
the first dataset's values are used for labels.
<p>
This chart is particularly useful for trends that don't have a strictly numeric or time oriented X axis.
You can define the axis start and end values, and by setting the "xAxisOptions" to include "noAutoScale", 
you can define the number of labels, ticks, and grids.  Since this approach gives you complete labelling 
and scaling control over the X axis, this chart is sometimes easier to use than "dateLineApp" 
(part of the timeseries collection) for creating
time oriented charts, as well.  This is especially true when your date/time values include missing data,
like weekends and evenings.
<p>
Notice the Y axis on this chart.  It uses percentage labels, in a format appropriate for your locale.  Any
line chart can use percentage or currency labels by adding the appropriate options to "yAxisOptions" or
"xAxisOptions".
</td></tr></table>
<hr>
<table><tr><td>
