<%@ taglib uri="/WEB-INF/jsp/kavachart-taglib.tld" prefix="chart" %>
<head>
<title>KavaChart Area Chart Samples</title>
<link rel="stylesheet" type="text/css" href="../../../documentation/style.css" />
</head>
<font face="Arial,Helvetica" />
<P>
Area charts are ideal for showing trends, especially trends that 
consolidate two or more variables.  "dateAreaApp" automatically places labels at meaningful increments,
like months, weeks, hours, etc.
<p>
KavaChart ProServe's Timeseries collection uses the object "dateAreaApp"
to create timeseries area charts.
<p>
Use the source code to this page as a guide in creating your own area charts.
You can also use the KavaChart <a href="http://www.ve.com/editor/" target="_blank" />
ChartWizard</a>
to design your own area charts.
<hr>
<table border=0><tr><td valign=top><tr><td valign=top>
The "dateAreaApp" object is ideal for time series that span
many observations, such as this tidal prediction plot.
<p>
Since our data isn't stacked, we can use a "baseline" value of "0.0" to provide a visual 
representation of plus and minus tides.  A reference line supplies further information
about the mean tide.
<p>
DateAreaApp uses both X and Y values.  The X values are supplied as timestamps, using the
property "dataset0dateValues" for datasets 0 through 39, and Y values are supplied using the
property "dataset0yValues", for datasets 0 through 39.  Other approaches include using a DataProvider
or a web service to acquire the data.  These techniques are generally much better, but require a slightly 
deeper understanding of the page construction process.  See the tag and scriptlet examples for several
code samples you can adapt to your own data.
<p>
For these samples, we use the property
"customDatasetHandler", which reads comma-delimited columns of data, like
<a href="../data/tides.txt" />this file.</a>
<p>
The object interprets the incoming data using the property "inputDateFormat", which is
"yyyy-MM-dd  HH:mm" in this case to match our data file.  It's worth thinking about how
you want to supply this information: in the timezone of the viewer, or the timezone of
the local event.  In our case, we're using values from Pacific Daylight time, since
that's the location for San Francisco's tides.  If you're in Tokyo or Brussels, however,
the times will obviously be incorrect.
<p>
You can adjust timezones using the "timezone" property.
</td><td valign=top>
<chart:streamed chartType="dateAreaApp" >
	<chart:param name="width" value="500" />
	<chart:param name="height" value="320" />
	<chart:param name="titleString" value="Tidal Information For San Francisco, CA" />
	<chart:param name="subTitleString" value="4 August 2003 - 11 August 2003" />
	<chart:param name="yAxisOptions" value="gridOn,rightAxis,rotateTitle,minTickOn" />
	<chart:param name="yAxisTitle" value="Tide In Feet" />
	<chart:param name="xAxisOptions" value="gridOn" />
	<chart:param name="baseline" value="0.0" />
	<chart:param name="inputDateFormat" value="yyyy-MM-dd  HH:mm" />
	<chart:param name="customDatasetHandler" value="<%=application.getRealPath("/examples/samples/data/tides.txt")%>" />
	<chart:param name="dataset0Color" value="006500" />
	<chart:param name="plotAreaColor" value="ccccee" />
	<chart:param name="plotAreaLeft" value="0.05" />
	<chart:param name="titleX" value="0.05" />
	<chart:param name="subTitleX" value="0.05" />
	<chart:param name="plotAreaTop" value="0.85" />
	<chart:param name="dwellYString" value="Tide: XX ft" />
	<chart:param name="yAxisThresholdLine0Value" value="3.1" />
	<chart:param name="yAxisThresholdLine0LabelString" value="Mean Tide" />
	<chart:param name="yAxisThresholdLine0Color" value="yellow" />
	<chart:param name="yAxisThresholdLine0LabelColor" value="yellow" />
	<chart:param name="dwellXString" value="XX" />
</chart:streamed>
</td></tr></table>
<hr>
<table border=0><tr><td valign=top><tr><td valign=top>
<chart:streamed chartType="dateAreaApp" >
	<chart:param name="width" value="500" />
	<chart:param name="height" value="320" />
	<chart:param name="customDatasetHandler" value="<%=application.getRealPath("/examples/samples/data/tahoe.txt")%>" />
	<chart:param name="titleString" value="USGS Water Elevations in Lake Tahoe, California" />
	<chart:param name="titleFont" value="Arial,14,1" />
	<chart:param name="dataset0Color" value="008800" />
	<chart:param name="dataset0SecondaryColor" value="004400" />
	<chart:param name="dataset0Gradient" value="2" />
	<chart:param name="inputDateFormat" value="yyyy-MM-dd HH:mm" />
	<chart:param name="axisDateFormat" value="MMM-dd" />
	<chart:param name="dwellLabelDateFormat" value="MMM-dd" />
	<chart:param name="yAxisOptions" value="gridOn,minTickOn,rotateTitle" />
	<chart:param name="yAxisTitle" value="Water Elevation" />
	<chart:param name="xAxisOptions" value="gridOn,minTickOn" />
	<chart:param name="plotAreaRight" value="0.92" />
	<chart:param name="plotAreaLeft" value="0.15" />
	<chart:param name="plotAreaTop" value="0.90" />
	<chart:param name="plotAreaBottom" value="0.19" />
	<chart:param name="yAxisEnd" value="5.0" />
	<chart:param name="yAxisStart" value="4.0" />
	<chart:param name="yAxisLabelPrecision" value="2" />
	<chart:param name="dwellYString" value="Gauge Reading: XX inches" />
	<chart:param name="dwellXString" value="XX" />
</chart:streamed>

</td><td valign=top>
<p>
Like other KavaChart objects, you can use "axisStart" and "axisEnd" to provide "hints" about
how the axis should scale.  In the case of time oriented axes, use "startDate", and "endDate"
to provide these hints.  You can also supply a "scalingType" to indicate whether increments
should occur at minutes, hours, days, etc.
<p>
Since the lake water levels stay relatively constant in this chart's display period,
we set the Y axis start and end range from 4.0 to 5.0.  The default tiny range (4.5 to 4.7)
unduly exaggerates the trend.
<p>
This chart also reads data from a "customDatasetHandler", to keep the source code to this page
brief, but you could easily embed the time stamps and gauge observations into the
properties dataset0dateValues and dataset0yValues.

</td></tr></table>
<hr>
