<!--
  A JSP that also starts a cache cleaner instance.
-->

<jsp:useBean id="chart" class="com.ve.kavachart.servlet.barApp" scope="page"/>

<html>
<head><title>Cache Cleaner Example</title>
	<link type="text/css" rel="stylesheet" href="../../../documentation/style.css">
</head>
<body bgcolor="white">

<% 
	//add some random data here:
	for(int i=0;i<5;i++){
		double d = Math.random();
		chart.accumulateProperty("dataset0yValues", Double.toString(d));
	}

	//set a few other properties to make the chart look nice
	chart.setProperty("colorPalette", "web_seattle");
	chart.setProperty("individualColors", "true");
	chart.setProperty("titleString", "hello, world");
	chart.setProperty("writeDirectory", application.getRealPath("images/KavaChartImages/"));
	chart.setProperty("yAxisOptions", "gridOn, minTickOn");
	chart.setProperty("xAxisOptions", "gridOn");
	chart.setProperty("dwellLabelTemplate", "Value: <y>");


    //activate the cache cleaner and set a few properties
    chart.setProperty("useCacheCleaner", "true");
    chart.setProperty("cacheCleanerDirectory", application.getRealPath("images/KavaChartImages/")); // this must match the writeDirectory
    // be careful - almost everything in that directory will be erased!!!
    chart.setProperty("cacheCleanerInterval", "10"); // every 10 minutes, CacheCleaner will run
    chart.setProperty("cacheCleanerExpirationTime", "5"); //everything older than five minutes will be deleted
%>
<table><tr><td><img src=../../../images/KavaChartImages/<%= chart.getFileName() %>></td>
<td>

<p>
A very simple chart example, with a difference:
<p>
You may have noticed that almost all the demonstration JSPs write an image into the
"cache" directory.  If you're on a high-volume server, the cache directory will
quickly fill with small image files.
</td></tr></table>
<p>
By setting the property "useCacheCleaner" to "true", and setting some other properties,
you can start a thread that will empty the cache directory periodically.
<pre>
    //activate the cache cleaner and set a few properties
    chart.setProperty("useCacheCleaner", "true");
    chart.setProperty("cacheCleanerDirectory", application.getRealPath("images/KavaChartImages/")); // this must match the writeDirectory
    // be careful - almost everything in that directory will be erased!!!
    chart.setProperty("cacheCleanerInterval", "10"); // every 10 minutes, CacheCleaner will run
    chart.setProperty("cacheCleanerExpirationTime", "5"); //everything older than five minutes will be deleted
</pre>
<p>
Whenever the KavaChart cache cleaner runs, it clears all files older than a specified
time from the cache directory.  The run interval and the expiration time will vary,
depending on your application.  Some applications can reuse charts for days, or even
months.  Others require more frequent maintenance.  This example runs every ten minutes.
If you examine the cache directory in another 10 minutes, you'll see
that all image files older than 5 minutes have been removed.
<p>
Since you don't want to start a new thread for every image, KavaChart uses a single
CacheCleaner for all charts.  If you're using multiple cache directories, start
CacheCleaner independently, rather than using chart properties.
<p>
<em>Note:</em>
Another approach is to use ProServe chart tags, which will stream images to your page without saving them
on the server.
</body>
</html>
