<html>
<!--
  Copyright (c) 2004 Visual Engineering, Inc.  All rights 
  reserved.
-->
<head>
	<title>KavaChart ProServe - Persistent Data, Histogram Data Handler</title><p>
	<link type="text/css" rel="stylesheet" href="../../documentation/style.css">
</head>
<body>
<%@ taglib uri="/WEB-INF/jsp/kavachart-taglib.tld" prefix="chart" %>
<jsp:useBean id="buoy" scope="application" class="examples.MontereyBouyData" />
<p>
<a href="http://www.noaa.gov">
<img src="http://www.noaa.gov/images/head_title.png">
</a>

<%
	//handle form information:
	int numberOfDays = 9; //our default
	int endHour = 1100; //we get this much data from the buoy
	String requestedDays = request.getParameter("days");
	if(requestedDays!=null)
		numberOfDays = Integer.parseInt(requestedDays);
	int startHour = endHour - numberOfDays * 24;
%>
<br>
<p>
This page acquires real time weather information
from a buoy in the Pacific Ocean off the central California coast.  Since the data acquisition
process is lengthy and the data only changes every hour, we don't want to re-read the data every time
the page is refreshed.  By storing our data in an "application" scope bean named "buoy",
our DataProvider is available to every page in the application, and it won't contact the
NOAA web site until the DataProvider's timeout (1 hour) has been reached.
<p>
The "buoy" DataProvider stores a variety of information, including wind, temperature, and wave
height data.  We use "datafilter" tags to split the data up into various charts, and to narrow
the number of days we're looking at.

<center>
<table>

<tr><td>
Wind Information
<p>
If wind speeds exceed 20 MPH, you should see a reference line that indicates a gale warning.
</td><td>
<chart:cached
	style="styles/buoy.properties" 
	chartType="disDateLineApp" 
	useLinkMap="false" >

	<chart:datafilter 
		dataProviderID="buoy" 
		startDataset="1" 
		endDataset="1" 
		startObservation="<%=startHour%>">
	</chart:datafilter>
	<chart:param name="yAxisThresholdLine0Color" value="b8bc9c" />
	<chart:param name="yAxisThresholdLine0LabelColor" value="999966" />
	<chart:param name="yAxisThresholdLine0LabelFont" value="SansSerif,11,0" />
	<chart:param name="yAxisThresholdLine0LabelString" value="Storm Level" />
	<chart:param name="yAxisThresholdLine0Value" value="20.0" />

</chart:cached>
</td></tr>

<tr><td>
Temps
<p>
This buoy seems to have intermittent temperature data.  Missing values are indicated by
a broken line.
</td><td>
<chart:cached
	style="styles/buoy.properties" 
	chartType="disDateLineApp" 
	useLinkMap="false" >

	<chart:datafilter 
		dataProviderID="buoy" 
		startObservation="<%=startHour%>"
		startDataset="3" 
		endDataset="4">
	</chart:datafilter>

</chart:cached>
</td></tr>

<tr><td>
Wave Heights
<p>
When wave heights at this buoy exceed 10 feet, you probably don't want to be
in the water.  A storm level warning reference line should appear on the chart.
</td><td>
<chart:cached
	style="styles/buoy.properties" 
	chartType="disDateLineApp" 
	useLinkMap="false" >

	<chart:datafilter 
		dataProviderID="buoy" 
		startObservation="<%=startHour%>"
		startDataset="2" 
		endDataset="2">
	</chart:datafilter>
	<chart:param name="yAxisThresholdLine0Color" value="b8bc9c" />
	<chart:param name="yAxisThresholdLine0LabelColor" value="999966" />
	<chart:param name="yAxisThresholdLine0LabelFont" value="SansSerif,11,0" />
	<chart:param name="yAxisThresholdLine0LabelString" value="Storm Level" />
	<chart:param name="yAxisThresholdLine0Value" value="10.0" />

</chart:cached>
</td></tr>

<tr><td>
Wave Height Distribution
<p>
By wrapping our data source in the KavaChart ProServe "datahistogram" tag, we can do a quick
ad-hoc visual analysis of wave heights in California's Monterey Bay.  The Y axis tells us
how many observations are in each "bin", while the X axis label tells us the mean height
of that particular bin.
<p>
The chart usually forms a natural looking bell curve if you have enough observations, since
we're looking at natural data.
</td><td>
<chart:cached
	style="styles/buoy.properties" 
	chartType="columnApp" 
	useLinkMap="false" >

	<chart:datahistogram>
		<chart:datafilter 
			dataProviderID="buoy" 
			startObservation="<%=startHour%>"
			startDataset="2" 
			endDataset="2">
		</chart:datafilter>
	</chart:datahistogram>
	<chart:param name="dataset0Name" value="Wave Height Histogram" />
</chart:cached>
</td></tr>

</table>
</center>
<br>
<p>
In addition to optimizing our data sources, these charts are stored on the server using the ProServe
"cached" tag.  That means the charts won't be regenerated if they already exist at the server.  While
this probably isn't a big optimization for a demonstration application, if you have thousands of
users viewing your pages, the savings in server resources can be significant.
<p>
Try changing the number of days with the form below to see what the performance is like, re-calculating
data sources, rendering new charts, evaluating a new histogram, etc.
<p>
<form method=post action="buoy.jsp">
Number of Days Data:

<select name="days">
<%
	for(int i=1;i<12;i++){
		out.print("<option value=" + i);
		if(i==numberOfDays)
			out.print(" selected");
		out.print(">" + i);
	}
%>
</select>

<INPUT type=submit value="Retrieve Data">
</form>
</body>
</html>
